import json
import os

from openxlab.config import const


def get_token() -> str:
    raise Exception(
        "This method has been discarded, please use get_jwt method in openxlab.xlab.handler.user_token to "
        "get token, see ReadMe for details"
    )


def check_login():
    """check user login or not. If user logged in, return true, else return false. Used in ls or info dataset repository command"""
    try:
        config_path = os.path.join(const.DEFAULT_CONFIG_DIR, const.DEFAULT_CLI_CONFIG_FILE_NAME)
        if os.path.exists(config_path):
            with open(config_path, encoding='utf-8') as f:
                config_content = f.read()
            config_dict = json.loads(config_content)
            if config_dict['ak'] and config_dict['sk']:
                return True
    except Exception:
        pass
    return False



""" 
get the file list of the dataset repository
"""
from openxlab.dataset.handler.list_dataset_repository import query
from openxlab.types.command_type import *


class Ls(BaseCommand):
    """List dataset repository resources."""

    def get_name(self) -> str:
        return "ls"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = (
            "openxlab dataset ls [OPTIONS]\n\n"
            "List dataset repository resources. \nNote: if you are not log in, you can only get the list of public dataset repository.\n\n"
            "Example:\n"
            "> openxlab dataset ls --dataset-repo \"username/dataset-repo-name\""
        )
        parser.add_argument(
            "-r",
            "--dataset-repo",
            type=str,
            required=True,
            help="The address of dataset repository. format: username/dataset-repo-name.[required]",
        )

    def take_action(self, parsed_args: Namespace) -> int:
        query(dataset_repo=parsed_args.dataset_repo)
        return 0



""" 
create new dataset repository 
"""

from rich import print as rprint

from openxlab.dataset.commands.utility import ContextInfo
from openxlab.types.command_type import *


def create_repo(repo_name: str, private: bool = False):
    """
    Create a dataset repository.

    Example:
        openxlab.dataset.create_repo(repo_name="dataset_repo_name")

    Parameters:
        @repo_name String The name of dataset repository.
    """
    ctx = ContextInfo()
    client = ctx.get_client()

    permission = 'private' if private else 'public'
    req_data_dict = {"name": f"{repo_name}", "displayname": f"{repo_name}", "state": permission}

    resp_data_dict = client.get_api().create_dataset(req=req_data_dict)

    rprint(
        f"Dataset named: [blue]{resp_data_dict['name']}[/blue] created successfully. Dataset is {permission}."
    )



import hashlib
import io
import multiprocessing
import os
from pathlib import Path
from typing import List
from typing import Optional
from typing import Tuple
from urllib.parse import urljoin

import requests
from rich import print as rprint
from tqdm import tqdm

from openxlab.dataset.utility.concurrency import concurrent_submit
from openxlab.types.command_type import *


# buf_size = 128 kb chunksize
def calculate_file_sha256(file_path: str, buf_size: int = 131072):
    if not Path(file_path).is_file():
        raise Exception(f"file {file_path} does not exist")
    sha256_obj = hashlib.sha256()
    with open(file_path, 'rb') as f:
        while True:
            data = f.read(buf_size)
            if not data:
                break
            sha256_obj.update(data)

    return sha256_obj.hexdigest()


class FileInfo:
    def __init__(self, client, abs_path: str, rel_path: Optional[str] = None):
        self.abs_path = abs_path
        self.rel_path = rel_path
        self.sha256: Optional[str] = None
        self.size: Optional[int] = None
        self.pre_upload: Optional[Dict] = None
        self.upload_process: Optional[List[Dict]] = []
        self.dataset: Optional[str] = None
        self.branch: Optional[str] = None
        self.sink_path: Optional[str] = None
        self.upload_id: Optional[str] = None
        self.client = client

    def get_size(self):
        self.size = os.stat(self.abs_path).st_size

    def get_pre_upload_info(self, dataset, branch, sink_path: Optional[str] = None):
        self.dataset = dataset
        self.branch = branch

        if self.size is None:
            self.get_size()
        if self.sha256 is None:
            self.sha256 = calculate_file_sha256(self.abs_path)
        if self.rel_path is None:
            self.rel_path = os.path.split(self.abs_path)[-1]

        if self.sink_path is None:
            self.sink_path = self.rel_path if sink_path is None else sink_path
        req_dict = {"size": self.size, "sha256": self.sha256}
        self.pre_upload = self.client.pre_object_upload(
            self.dataset, self.branch, self.sink_path, req_dict
        )
        if not self.pre_upload["exists"]:
            self.upload_id = self.pre_upload["id"]

    def put_part(self, part_number: int):
        if self.upload_id is None:
            raise Exception(f"no upload id info")

        part_number_list = [part["number"] for part in self.pre_upload["parts"]]
        if part_number not in part_number_list:
            raise Exception(f"the part number does not exist")

        part = [part for part in self.pre_upload["parts"] if part["number"] == part_number][0]
        part_put_url = part["putUrl"]
        part_size = self.pre_upload["partSize"]
        if part["number"] == 0:
            data = Path(self.abs_path).open("rb")
        else:
            offset = (part_number - 1) * part_size
            read_size = min(self.size - offset, part_size)
            with open(self.abs_path, "rb") as f:
                f.seek(offset)
                data = io.BufferedReader(io.BytesIO(f.read(read_size)))

        put_resp = requests.put(url=part_put_url, data=data)
        data.close()

        if put_resp.status_code != 200:
            raise Exception(
                f"put failed, status_code = {put_resp.status_code}, text = {put_resp.text}"
            )
        etag = put_resp.headers['Etag']
        part_submit_dict = {"number": part_number, "etag": etag}
        self.upload_process.append(part_submit_dict)

    def get_post_upload_info(self):
        if not self.upload_process or len(self.upload_process) != len(self.pre_upload["parts"]):
            raise Exception("no upload task or upload does not finish")

        post_req = {"id": self.upload_id, "parts": self.upload_process}
        post_resp = self.client.post_object_upload(
            self.dataset, self.branch, self.sink_path, post_req
        )
        if post_resp["sha256"] != self.sha256:
            raise Exception(f"file sha256 mismatch!")


def put_file_part(file: FileInfo, part_number: int):
    if file.upload_id is None:
        raise Exception(f"no upload id info")

    part_number_list = [part["number"] for part in file.pre_upload["parts"]]
    if part_number not in part_number_list:
        raise Exception(f"the part number does not exist")

    part = [part for part in file.pre_upload["parts"] if part["number"] == part_number][0]
    part_put_url = part["putUrl"]
    part_size = file.pre_upload["partSize"]
    if part["number"] == 0:
        data = Path(file.abs_path).open("rb")
    else:
        offset = (part_number - 1) * part_size
        read_size = min(file.size - offset, part_size)
        with open(file.abs_path, "rb") as f:
            f.seek(offset)
            data = io.BufferedReader(io.BytesIO(f.read(read_size)))

    put_resp = requests.put(url=part_put_url, data=data)
    data.close()

    if put_resp.status_code != 200:
        raise Exception(
            f"put failed, status_code = {put_resp.status_code}, text = {put_resp.text}"
        )
    etag = put_resp.headers['Etag']
    part_submit_dict = {"number": part_number, "etag": etag}
    file.upload_process.append(part_submit_dict)


def _get_files_in_folder(client, folde_path: str, mts: bool = False) -> List[FileInfo]:
    if not Path(folde_path).is_dir():
        raise Exception(f"path {folde_path} is not a dir or does not exist")
    root_path = os.path.abspath(folde_path)
    res_list = multiprocessing.Manager().list() if mts else []
    for root, _, files in os.walk(root_path):
        for file in files:
            file_path = os.path.join(root, file)
            rel_path = os.path.relpath(file_path, root_path).replace("\\", "/")
            file_obj = FileInfo(client, file_path, rel_path)
            file_obj.get_size()
            res_list.append(file_obj)
    return res_list


def update_progress(update_value: int, progress: Optional[tqdm] = None):
    if progress:
        progress.update(update_value)


def upload_files_worker(
    task_list: List[Tuple[FileInfo, int, int]], progress: Optional[tqdm] = None
):
    while len(task_list) > 0:
        task = task_list.pop(0)
        file = task[0]
        part_number = task[1]
        part_size = task[2]
        if file.upload_id is None:
            update_progress(file.size, progress)
        else:
            # put_file_part(file, part_number)
            file.put_part(part_number)
            if len(file.pre_upload["parts"]) == len(file.upload_process):
                file.get_post_upload_info()
            update_progress(part_size, progress)


def upload_files(
    task_list: List[Tuple[FileInfo, int, int]], progress: Optional[tqdm] = None, workers: int = 8
):
    concurrent_submit(upload_files_worker, workers, task_list, progress)


def files_to_tasks(
    file_list: List[FileInfo],
    task_list: List[Tuple[FileInfo, int, int]],
    dataset: str,
    branch: str,
    progress: Optional[tqdm] = None,
):
    while len(file_list) > 0:
        file = file_list.pop(0)
        file.get_pre_upload_info(dataset, branch)

        if file.upload_id:
            if len(file.pre_upload["parts"]) == 1:
                task = (file, 0, file.size)
                task_list.append(task)
            else:
                for part in file.pre_upload["parts"]:
                    part_number = part["number"]
                    size = min(
                        file.pre_upload["partSize"],
                        file.size - (part_number - 1) * file.pre_upload["partSize"],
                    )
                    task_list.append((file, part_number, size))
        else:
            task = (file, 0, 0)
            task_list.append(task)
        update_progress(1, progress)


def get_task_list(
    file_list: List[FileInfo],
    dataset: str,
    branch: str,
    progress: Optional[tqdm] = None,
    workers: int = 8,
):
    task_list = []
    concurrent_submit(files_to_tasks, workers, file_list, task_list, dataset, branch, progress)
    return task_list


# 获取dataset id的方法，此处先做测试
def get_dataset_id(client, dataset_repo_name: str):
    dataset_id = str(client.get_dataset_info(dataset_repo_name)["id"])
    return dataset_id


class Uploader:
    def __init__(self, client, dataset_repo_name: str):
        self.dataset = dataset_repo_name
        self.branch = "main"  # TODO default to main this version
        self.buffer_size = 128 * 1024  # 128 kb
        self.workers = 8
        self.client = client

    def upload_folder(self, source_path: str, destination_path: str):
        if not destination_path:
            destination_path = "/"

        if not destination_path.endswith("/"):
            destination_path = destination_path + "/"
        file_list = _get_files_in_folder(self.client, source_path)
        for file in file_list:
            file.sink_path = urljoin(destination_path, file.rel_path)

        total_num = len(file_list)
        total_size = sum([file.size for file in file_list])

        with tqdm(total=total_num, ncols=100, desc="Preparing", unit_scale=True) as progress:
            task_list = get_task_list(file_list, self.dataset, self.branch, progress, self.workers)

        with tqdm(total=total_size, ncols=100, desc="Uploading", unit_scale=True) as progress:
            upload_files(task_list, progress, self.workers)
        # rprint("Upload folder successfully!")

    def upload_file(self, source_path: str, destination_path: str):
        if not os.path.isfile(source_path):
            raise Exception(f"{source_path} is not a file or does not exist!")
        if not destination_path:
            destination_path = "/"
        if not destination_path.endswith("/"):
            destination_path = destination_path + "/"

        file_obj = FileInfo(self.client, source_path)
        file_obj.get_size()
        file_obj.sink_path = urljoin(destination_path, os.path.split(source_path)[-1])
        print(urljoin(destination_path, os.path.split(source_path)[-1]))
        print("checking file info...")
        file_list = [file_obj]
        task_list = []

        total_size = file_obj.size
        files_to_tasks(file_list, task_list, self.dataset, self.branch)
        with tqdm(total=total_size, ncols=100, desc="Uploading", unit_scale=True) as progress:
            upload_files(task_list, progress, self.workers)

        # rprint("Upload file successfully!")



from concurrent.futures import ThreadPoolExecutor
from concurrent.futures import wait
from typing import Callable
from typing import Iterable


def concurrent_submit(func: Callable, workers: int, *args):
    with ThreadPoolExecutor(max_workers=workers) as executor:
        pool = [executor.submit(func, *args) for i in range(workers)]
        results = [f.result() for f in pool]
        return results


def concurrent_map(func: Callable, workers: int, iters: Iterable):
    with ThreadPoolExecutor(max_workers=workers) as executor:
        results = executor.map(func, iters)
        return results


if __name__ == "__main__":
    print("+++++++++++++++ test ++++++++++++++++++")
    t_list = [1, 2, 3, 4, 5]
    concurrent_map(lambda x: print(x + 1), 5, t_list)
    print(t_list)



""" 
remove the dataset repository
"""
from argparse import ArgumentParser
from argparse import Namespace

from openxlab.dataset.handler.remove_dataset_repository import remove_repo
from openxlab.types.command_type import *


class Remove(BaseCommand):
    """Remove a dataset repository."""

    def get_name(self) -> str:
        return "remove"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = (
            "openxlab dataset remove [OPTIONS]\n\n"
            "Remove a dataset repository.\n\n"
            "Example:\n"
            "> openxlab dataset remove --dataset-repo \"username/dataset-repo-name\""
        )
        parser.add_argument(
            "-r",
            "--dataset-repo",
            help="The address of dataset repository. format: username/dataset-repo-name.[required]",
        )

    def take_action(self, parsed_args: Namespace) -> int:
        remove_repo(dataset_repo=parsed_args.dataset_repo)

        return 0



""" 
delete dataset repository for user
"""
from rich import print as rprint

from openxlab.dataset.commands.utility import ContextInfo
from openxlab.types.command_type import *


def remove_repo(dataset_repo: str):
    """
    Remove a dataset repository.

    Example:
        openxlab.dataset.remove_repo(dataset_repo="username/dataset_repo_name")

    Parameters
        @dataset_repo String The address of dataset repository.
    """
    ctx = ContextInfo()
    client = ctx.get_client()
    parsed_ds_name = dataset_repo.replace("/", ",")
    rprint(f"Removing dataset repository [red]{dataset_repo}[/red]...")
    client.get_api().delete_repo(parsed_ds_name)
    rprint(f"Dataset repository [blue]{dataset_repo}[/blue] removed successfully.")



""" 
query to get the information of dataset repository
"""
from rich import box
from rich.console import Console
from rich.table import Table

from openxlab.dataset.commands.utility import ContextInfoNoLogin


def info(dataset_repo: str):
    """
    Get the information of a dataset repository.
    Note: if you are not log in, you can only get the information of public dataset repository.

    Example:
        openxlab.dataset.info(dataset_repo="username/dataset_repo_name")

    Parameters:
        @dataset_repo String The address of dataset repository.
    """
    ctx = ContextInfoNoLogin()
    client = ctx.get_client()

    parsed_ds_name = dataset_repo.replace("/", ",")

    info_data = client.get_api().get_dataset_info(dataset_name=parsed_ds_name)
    info_data_result = _reformat_info_data(info_data)
    console = Console()
    table = Table(show_header=True, header_style='bold cyan', box=box.ASCII2)
    table.add_column("Field", width=20, justify='full', overflow='fold')
    table.add_column("Content", width=120, justify='full', overflow='fold')

    for key in info_data_result.keys():
        val = info_data_result[key]
        val = "" if not val else val
        table.add_row(key, val, end_section=True)

    console.print(table)

    return info_data_result


def _format_types(info_data, type_name):
    types_str = ""
    if type_name in info_data['attrs'].keys():
        types_list = info_data['attrs'][type_name]
        if types_list and len(types_list) > 0:
            types_str = ", ".join([x['name']['en'] for x in types_list])

    return types_str


def _reformat_info_data(info_data):
    publisher_str = _format_types(info_data, 'publisher')
    media_types_str = _format_types(info_data, 'mediaTypes')
    label_types_str = _format_types(info_data, 'labelTypes')
    task_types_str = _format_types(info_data, 'taskTypes')
    if info_data['introduction']:
        data_introduction = info_data['introduction']['en']
    else:
        data_introduction = ""
    introduction_str = ""
    if data_introduction and len(data_introduction) > 0:
        introduction_str = data_introduction[:97] + '...'

    info_data_result = {
        'Name': info_data['name'],
        'Introduction': introduction_str,
        'Author': publisher_str,
        'Data Type': media_types_str,
        'Label Type': label_types_str,
        'Task Type': task_types_str,
    }

    return info_data_result



""" 
upload folder to dataset repository
"""
from openxlab.dataset.handler.upload_dataset_folder import upload_folder
from openxlab.types.command_type import *


class UploadFolder(BaseCommand):
    """Upload folder from local to remote."""

    def get_name(self) -> str:
        return "upload-folder"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = (
            "openxlab dataset upload-folder [OPTIONS]\n\n"
            "Upload folder from local to remote.\n\n"
            "Example:\n"
            "> openxlab dataset upload-folder  --dataset-repo \"username/dataset-repo-name\" --source-path \"/path/to/local/folder\" --target-path \"/raw/folder\""
        )
        parser.add_argument(
            "-r",
            "--dataset-repo",
            type=str,
            required=True,
            help="The address of dataset repository. format: username/dataset-repo-name.[required]",
        )
        parser.add_argument(
            "-s",
            "--source-path",
            type=str,
            required=True,
            help=("The local path of the folder to upload.[required]"),
        )
        parser.add_argument(
            "-t",
            "--target-path",
            type=str,
            # required=True,
            help=("The target path to upload folder.[optional]"),
        )

    def take_action(self, parsed_args: Namespace) -> int:
        upload_folder(
            dataset_repo=parsed_args.dataset_repo,
            source_path=parsed_args.source_path,
            target_path=parsed_args.target_path,
        )

        return 0



from openxlab.dataset.client.api import XlabDatasetAPI


class Client:
    def __init__(self, host:str, odl_cookie:str):
        self.host = host
        self.odl_cookie = odl_cookie
        self.odl_api = None
        
    def get_api(self):
        self.odl_api = XlabDatasetAPI(host=self.host, cookie=self.odl_cookie)
        return self.odl_api


"""
Download function for related files in the reality model library
"""
import logging
import re
from tqdm import tqdm
import requests
import os
import hashlib
import sys
# import urllib.parse


from openxlab.model.clients.openapi_client import OpenapiClient
from openxlab.model.common.constants import endpoint, token, default_metafile_template_name, model_cache_path, \
    river_pass_url
from openxlab.model.common.bury import bury_data

logger = logging.getLogger("openxlab.model")


def download(model_repo, model_name=None, output=None, overwrite=False, ignore=None, cache=True) -> None:
    """
    download model file|meta file|log filee|readme file
    usage: cli & sdk
    model_repo: username/repository
    model_name: model name
    output: local storage path
    overwrite: whether to overwrite local files
    """
    try:
        # split params
        username, repository = _split_repo(model_repo)
        client = OpenapiClient(endpoint, token)
        if isinstance(model_name, str):
            model_name = [model_name]
        filepath = None
        models, files = client.get_download_url(username, repository, model_name, filepath, ignore)

    except ValueError as e:
        print(f"Error: {e}")
        return
    file_path_download = []
    cache_path = os.path.join(model_cache_path, f'{username}_{repository}')
    # 根据模型名称下载
    for i_name, i_model in models.items() if models is not None else []:
        url = i_model['url']
        file_name = i_model['fileName']
        _hash = i_model['hash']
        file_path = _download_to_local(url, file_name, output, overwrite, cache_path, allow_cache=cache)
        file_path_download.append(file_path)
    # 根据文件路径下载
    for i_name, i_file in files.items() if files is not None else []:
        url = i_file['url']
        file_name = i_file['fileName']
        _hash = i_file['hash']
        file_path = _download_to_local(url, file_name, output, overwrite, cache_path, allow_cache=cache)
        file_path_download.append(file_path)
    print("download model repo:{} file finished".format(model_repo))
    return file_path_download


@bury_data("init_metafile")
def download_metafile_template(path=None, all=False) -> None:
    """
    download metafile template file
    """
    try:
        # split params
        client = OpenapiClient(endpoint, token)
        url = client.get_metafile_template_download_url(all=all)
    except ValueError as e:
        print(f"Error: {e}")
        return
    _download_to_local(url, file_name=default_metafile_template_name, path=path, allow_cache=False)


def download_from_url(url, path=None, overwrite=False, file_name=None):
    if file_name is None:
        file_name = os.path.basename(url)
    try:
        # url_encoded = urllib.parse.quote(url)
        # print(f'url encoded:{url_encoded}')
        river_pass_download_url = river_pass_url + '?url=' + url
        md5 = hashlib.md5(url.encode('utf-8'))
        url_hash = md5.hexdigest()
        path_file = _download_to_local(river_pass_download_url, file_name=file_name, path=path,
                                       overwrite=overwrite, allow_cache=True,
                                       cache_path=os.path.join(os.path.expanduser("~"), '.cache', 'riverpass',
                                                               url_hash))
    except Exception as e:
        print(f"use the original path to download")
        print(f"riverpass info:{e}")
        path_file = _download_to_local(url, file_name=file_name, path=path,
                                       overwrite=overwrite, allow_cache=True)

    print(f'success download file: {path_file}')
    return path_file


def _split_repo(model_repo) -> (str, str):
    """
    Split a full repository name into two separate strings: the username and the repository name.
    """
    # username/repository format check
    pattern = r'.+/.+'
    if not re.match(pattern, model_repo):
        raise ValueError("The input string must be in the format 'username/model_repo'")

    values = model_repo.split('/')
    return values[0], values[1]


@bury_data("download_file")
def _download_to_local(url, file_name, path=None, overwrite=False, cache_path=None, allow_cache=None) -> str:
    """
    download file to local with progress_bar
    url: file download url
    file_name: local file name
    path: local path, default current directory
    _hash: file hash
    weight_raw_size: file raw size
    overwrite: whether to overwrite local files
    cache_path: cache path, default is ~/.cache/model
    allow_cache: cache whether is allowed.
    """
    path_file = file_name
    cache_path_file = None
    if allow_cache is None or allow_cache:
        allow_cache = sys_allow_cache()
    if path is not None:
        path_file = os.path.join(path, file_name).replace('\\', '/')
    if allow_cache is True:
        if cache_path is None:
            cache_path = model_cache_path
        cache_path_file = os.path.join(cache_path, file_name).replace('\\', '/')
        if not os.path.exists(cache_path):
            os.makedirs(cache_path)
        elif os.path.exists(cache_path_file) and overwrite is not True:
            clear_and_link(cache_path_file, path_file, path)
            return path_file

    response = requests.get(url, stream=True)
    response.raise_for_status()
    total_size_in_bytes = int(response.headers.get('content-length', 0))
    block_size = 1024
    progress_bar = tqdm(total=total_size_in_bytes, unit='iB', unit_scale=True, desc=file_name)
    with open(cache_path_file if allow_cache else path_file, 'wb') as f:
        for data in response.iter_content(block_size):
            progress_bar.update(len(data))
            f.write(data)
    progress_bar.close()
    if allow_cache is True and os.path.exists(cache_path_file):
        clear_and_link(cache_path_file, path_file, path)
    return path_file


def get_file_hash(filename):
    hasher = hashlib.md5()
    with open(filename, 'rb') as file:
        file_size = file.seek(0, 2)
        if file_size <= 128:  # 如果文件小于等于128字节
            combined_block = file.read(file_size)  # 只读取一次，并将所有数据组合在一起
            hasher.update(combined_block)
        else:  # 否则，读取前64字节和后64字节
            file.seek(0)  # 将文件指针移动到文件开头
            first_block = file.read(64)
            hasher.update(first_block)
            file.seek(-64, 2)  # 将文件指针移动到文件末尾-64处
            last_block = file.read(64)
            hasher.update(last_block)
            combined_block = first_block + last_block
            hasher.update(combined_block)

    return hasher.hexdigest()


def clear_and_link(cache_path_file, path_file, path):
    if os.path.exists(path_file):
        os.remove(path_file)
    if os.path.islink(path_file):
        os.unlink(path_file)
    if path is not None and not os.path.exists(path):
        os.makedirs(path)
    os.symlink(cache_path_file, path_file)


def sys_allow_cache():
    """
    win disable cache
    """
    if sys.platform.startswith('win'):
        return False
    else:
        return True



"""
create model repository-cli
"""
from openxlab.types.command_type import *
from openxlab.model import create


class Create(BaseCommand):
    """Creating a model repository."""

    def get_name(self) -> str:
        return "create"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = 'openxlab model create [OPTIONS]\n' \
                       'Example:\n' \
                       '> openxlab model create --model-repo=\'username/model_repo_name\''
        parser.add_argument('-r', '--model-repo', required=True,
                            help='The name of model repository.[required]')
        parser.add_argument('-prt', '--private', type=bool, default=False,
                            help='BOOLEAN  The visibility of the model repository. Default is False.')
        parser.add_argument('-s', '--source', type=str, required=True,
                            help='The path of the meta file')
    def take_action(self, parsed_args: Namespace) -> int:
        create(parsed_args.model_repo, parsed_args.source, parsed_args.private)
        return 0



from openxlab.types.command_type import *
from openxlab.model.commands.download import Download
from openxlab.model.commands.wget import Wget
from openxlab.model.commands.upload import Upload
from openxlab.model.commands.init import Init
from openxlab.model.commands.create import Create
from openxlab.model.commands.list import List
from openxlab.model.commands.visibility import Visibility
from openxlab.model.commands.remove import Remove
from openxlab.model.commands.inference import Inference


class Model(BaseCommand):
    """model"""

    sub_command_list = [Upload, Download, Init, Create, List, Visibility, Remove, Inference, Wget]

    def get_name(self) -> str:
        return "model"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "--foo",
            type=str,
            help=(
                "this is an argument for test"
            ),
        )

    def take_action(self, parsed_args: Namespace) -> int:
        pass



from concurrent.futures import ThreadPoolExecutor


class BizThreadPool(object):
    def __init__(self, max_workers=5):
        self.executor = ThreadPoolExecutor(max_workers=max_workers)

    def submit_task(self, task, *args, **kwargs):
        future = self.executor.submit(task, *args, **kwargs)
        return future


# bury data thread pool
bury_thread_pool = BizThreadPool(5)



import oss2
import os
import sys

from oss2 import SizedFileAdapter
from oss2.exceptions import NoSuchUpload
from tqdm import tqdm

from openxlab.model.common.constants import oss_bucket_domain, oss_endpoint_domain, oss_accelerate_domain
from openxlab.utils.local_cache import cache


class OssClient(object):
    def __init__(self, endpoint, access_key_id, access_key_secret, security_token, bucket_name):
        self.bucket = self.get_bucket(endpoint, access_key_id, access_key_secret, security_token, bucket_name)
        self.cache_upload_id_key_prefix = f"model:oss:upload_id"

    def get_bucket(self, endpoint, access_key_id, access_key_secret, security_token, bucket_name):
        auth = oss2.StsAuth(access_key_id, access_key_secret, security_token)
        self.bucket = oss2.Bucket(auth, endpoint, bucket_name)
        return self.bucket

    def upload_to_oss(self, local_file_path, object_key, callback, callback_var):
        """
        simple upload
        """
        headers = {'Host': oss_bucket_domain, 'x-oss-callback': callback, 'x-oss-callback-var': callback_var}
        self.bucket.put_object_from_file(object_key, local_file_path, headers, progress_callback=percentage)

    def multipart_resume_upload(self, local_file_path, object_key, callback=None, callback_var=None, domain=None):
        headers = {'x-oss-callback': callback, 'x-oss-callback-var': callback_var}
        if domain == 'endpoint':
            headers['Host'] = oss_endpoint_domain
        elif domain == 'bucket':
            headers['Host'] = oss_bucket_domain
        elif domain == 'accelerate':
            headers['Host'] = oss_accelerate_domain

        total_size = os.path.getsize(local_file_path)
        part_size = oss2.determine_part_size(total_size, preferred_size=100 * 1024)
        upload_id = None
        cache_upload_id_key = f"{self.cache_upload_id_key_prefix}:{object_key}"
        try:
            upload_id = cache.get(cache_upload_id_key)
            uploaded_parts_numbers = []
            uploaded_parts = []
            if upload_id:
                uploaded_parts_result = self.bucket.list_parts(object_key, upload_id)
                if uploaded_parts_result:
                    uploaded_parts = uploaded_parts_result.parts
                    uploaded_parts_numbers = [part.part_number for part in uploaded_parts]
            else:
                # init upload
                upload_id = self.bucket.init_multipart_upload(object_key).upload_id
                cache.set(cache_upload_id_key, upload_id, expire=3600 * 24 * 7)
            # compute part count
            part_count = (total_size + part_size - 1) // part_size

            # begin multipart upload
            parts = [] + uploaded_parts
            with open(local_file_path, 'rb') as fileobj:
                progress_bar = tqdm(total=total_size, unit='iB', unit_scale=True)
                for i in range(part_count):
                    part_number = i + 1
                    if part_number in uploaded_parts_numbers:
                        continue
                    offset = i * part_size
                    num_to_upload = min(part_size, total_size - offset)
                    upload_part = self.bucket.upload_part(object_key, upload_id, part_number,
                                                          SizedFileAdapter(fileobj, num_to_upload))
                    parts.append(oss2.models.PartInfo(part_number, upload_part.etag))
                    progress_bar.update(offset + num_to_upload - progress_bar.n)
            progress_bar.close()
            # complete multipart upload
            self.bucket.complete_multipart_upload(object_key, upload_id, parts, headers)
            cache.delete(cache_upload_id_key)
            print(f'Successfully uploaded {object_key}')
        except Exception as e:
            print(f'Error uploading {object_key}: {e}')
            if isinstance(e, NoSuchUpload):
                # cancel upload task if error
                cache.delete(cache_upload_id_key)
                self.bucket.abort_multipart_upload(object_key, upload_id)
                print(f'Upload of {object_key} aborted')
            raise e


# consumed_bytes表示已上传的数据量。
# total_bytes表示待上传的总数据量。当无法确定待上传的数据长度时，total_bytes的值为None。
def percentage(consumed_bytes, total_bytes):
    if total_bytes:
        rate = int(100 * (float(consumed_bytes) / float(total_bytes)))
        print(f'{consumed_bytes}/{total_bytes} |  {rate}% has been uploaded!')
        sys.stdout.flush()



""" 
commit message of change of dataset repository
"""
from openxlab.dataset.commands.utility import ContextInfo


def commit(dataset_repo: str, commit_message: str):
    """
    Commit local changes.

    Example:
        openxlab.dataset.commit(dataset_repo="username/dataset_repo_name",commit_message="init")

    Parameters:
        @dataset_repo String The address of the dataset repository to commit.
        @commit_message String The detail message to commit of the repository.
    """
    ctx = ContextInfo()
    client = ctx.get_client()

    req_data_list = [f"{dataset_repo}", {"msg": f"{commit_message}"}]
    client.get_api().commit_dataset(req=req_data_list)



import requests as requests

from openxlab.xlab.common.response_dto import ReturnDto


def http_common_header():
    header_dict = {
        "Content-Type": "application/json",
        "accept": "application/json"
    }
    return header_dict


def valid_response_dto(response_dto, path, payload):
    if response_dto.msg_code != '10000':
        raise ValueError(f"call {path} error, message: {response_dto.msg}")
    response_data = response_dto.data
    if response_data is None or len(response_data) == 0:
        raise ValueError(f"call {path} error, message: {response_dto.msg}")
    if response_data['msgCode'] != '10000':
        raise ValueError(f"call {path} error, message: {response_data['msg']}")
    return response_data['data']


class AuthClient(object):
    def __init__(self, endpoint):
        self.endpoint = endpoint

    def http_post_response_dto(self, path, payload):
        headers = http_common_header()
        response = requests.post(f"{self.endpoint}{path}", json=payload, headers=headers)
        response.raise_for_status()
        response_dict = response.json()
        response_dto = ReturnDto.from_camel_case(response_dict)
        return response_dto

    def auth(self, ak):
        if len(ak) == 0:
            raise ValueError("access key must not be empty")
        path = "auth"
        payload = {"ak": ak}
        response_dto = self.http_post_response_dto(path, payload)
        auth_dict = valid_response_dto(response_dto, path, payload)
        return auth_dict['nonce'], auth_dict['algorithm']

    def get_jwt(self, ak, d):
        if len(ak) == 0 or len(d) == 0:
            raise ValueError("access key and d must not be empty")
        path = "getJwt"
        payload = {"ak": ak, "d": d}
        response_dto = self.http_post_response_dto(path, payload)
        jwt_dict = valid_response_dto(response_dto, path, payload)
        return jwt_dict

    def refresh_jwt(self, ak, refresh_token):
        if len(ak) == 0 or len(refresh_token) == 0:
            raise ValueError("access key and refresh_token must not be empty")
        path = "refreshJwt"
        payload = {"ak": ak, "refresh_token": refresh_token}
        response_dto = self.http_post_response_dto(path, payload)
        jwt_dict = valid_response_dto(response_dto, path, payload)
        return jwt_dict





from openxlab.config import version as config_version
from openxlab.dataset.commands.commit import Commit
from openxlab.dataset.commands.create import Create
from openxlab.dataset.commands.download import Download

# from openxlab.dataset.commands.download import Download
from openxlab.dataset.commands.get import Get
from openxlab.dataset.commands.info import Info
from openxlab.dataset.commands.ls import Ls
from openxlab.dataset.commands.remove import Remove
from openxlab.dataset.commands.upload_file import UploadFile
from openxlab.dataset.commands.upload_folder import UploadFolder
from openxlab.dataset.commands.visibility import Visibility
from openxlab.dataset.handler.commit_dataset_info import commit
from openxlab.dataset.handler.create_dataset_repository import create_repo
from openxlab.dataset.handler.download_dataset_repository import download
from openxlab.dataset.handler.get_dataset_repository import get
from openxlab.dataset.handler.info_dataset_repository import info
from openxlab.dataset.handler.list_dataset_repository import query
from openxlab.dataset.handler.remove_dataset_repository import remove_repo
from openxlab.dataset.handler.upload_dataset_file import upload_file
from openxlab.dataset.handler.upload_dataset_folder import upload_folder
from openxlab.dataset.handler.visible_dataset_repository import visibility
from openxlab.types.command_type import *


def help():
    print("help")


class Dataset(BaseCommand):
    """Dataset Demo"""

    sub_command_list = [
        Get,
        Create,
        UploadFile,
        UploadFolder,
        Info,
        Ls,
        Commit,
        Download,
        Remove,
        Visibility,
    ]

    def get_name(self) -> str:
        return "dataset"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = "openxlab dataset [OPTIONS]\n\n"
        # parser.add_argument(
        #     "--test",
        #     help=(" This argument is a test argument"),
        # )



"""
download model file|meta file|log file|readme file-cli
"""
from openxlab.types.command_type import *
from openxlab.model import download


class Download(BaseCommand):
    """Download model weight files."""

    def get_name(self) -> str:
        return "download"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = "openxlab model download [-h] -r MODEL_REPO -f FILE [-p PATH] [-o] {help} ...\n" \
                       'Example:\n' \
                       '> openxlab model download --model-repo=username/model_repo_name' \
                       '  --model_name=faster_rcnn \n' \
                       '  --output=/path/to/local/folder/\n' \
                       '  --overwrite=True'
        parser.add_argument('-r', '--model-repo', required=True,
                            help='model repository address. format:username/repository.')
        parser.add_argument('-m', '--model-name', required=False, nargs='+',
                            help='target model to be download.')
        parser.add_argument('-op', '--output', required=False,
                            help='setting download path.')
        parser.add_argument('-o', '--overwrite', default=False, type=bool,
                            help='force overwriting local files.')
        parser.add_argument('-i', '--ignore', required=False,
                            help='ignore is used to filter files that do not need to be downloaded.')

    def take_action(self, parsed_args: Namespace) -> int:
        download(parsed_args.model_repo, model_name=parsed_args.model_name, output=parsed_args.output,
                 overwrite=parsed_args.overwrite, ignore=parsed_args.ignore)
        return 0



"""
remove repository
"""
from openxlab.types.command_type import *
from openxlab.model import remove


class Remove(BaseCommand):
    """Remove the model repository."""

    def get_name(self) -> str:
        return "remove"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = 'openxlab model remove [-h] -r MODEL_REPO {help} ...\n' \
                       'Example:\n' \
                       '> openxlab model remove --model-repo=username/model_repo_name'
        parser.add_argument('-r', '--model-repo', required=True,
                            help='The name of model repository.[required]')

    def take_action(self, parsed_args: Namespace) -> int:
        remove(parsed_args.model_repo)
        return 0



"""
model repository init-cli
"""
from openxlab.types.command_type import *
from openxlab.model.handler.download_file import download_metafile_template


class Init(BaseCommand):
    """Initialize the generated metafile.yaml file."""

    def get_name(self) -> str:
        return "init"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = 'openxlab model init [OPTIONS]\n' \
                       'Example:\n' \
                       '> openxlab model init --path=\'/local/to/path\''
        parser.add_argument('-p', '--path', required=False,
                            help='The path where the generated files are stored.')
        parser.add_argument('-a', '--all', default=False, required=False,
                            help='The path where the generated files are stored.')
    def take_action(self, parsed_args: Namespace) -> int:
        download_metafile_template(parsed_args.path, parsed_args.all)
        return 0




# from distutils.command.config import config
from functools import wraps
import json
import sys
from typing import Any
from typing import Callable
from typing import TypeVar

import requests

from openxlab.dataset.client.client import Client
from openxlab.dataset.client.uaa import get_odl_token
from openxlab.dataset.commands.config import config as client_config
from openxlab.dataset.constants import endpoint
from openxlab.dataset.exception import *
from openxlab.dataset.exception import OpenDataLabError
from openxlab.xlab.handler.user_config import get_config


_Callable = TypeVar("_Callable", bound=Callable[..., None])


class ContextInfo:
    """This class contains command context."""

    def __init__(self):
        self.credent = get_config(need_force=True)
        self.ak = self.credent.ak
        self.sk = self.credent.sk
        self.url = endpoint
        # get config of dataset.json
        self.config = client_config
        self.conf_file = client_config._get_config_filepath()
        self._conf_content = self.check_config()
        # print(self._conf_content)
        if "ssouid" not in self._conf_content:
            config_json = self.odl_auth()
            self.update_config(config_json)
            self._conf_content = self.check_config()
        odl_cookie = self._conf_content
        self.cookie = odl_cookie
        self.check_ret = 0

    def odl_auth(self):
        code, sso_uid = get_odl_token(self.ak, self.sk)
        data = {
            "code": code,
            "redirect": "",
        }
        data = json.dumps(data)

        resp = requests.post(
            f"{self.url}/datasets/api/v2/users/auth",
            data=data,
            headers={"Content-Type": "application/json"},
        )
        if resp.status_code != 200:
            raise OdlAuthError(resp.status_code, resp.text)

        odl_token = resp.json()["data"]["token"]
        config_json = {'opendatalab_session': odl_token, 'ssouid': sso_uid}
        return config_json

    def get_client(self) -> Client:
        return Client(host=self.url, odl_cookie=self.cookie)

    def get_content(self):
        return self._conf_content

    def set_content(self, content: dict) -> None:
        for key, value in content.items():
            self._conf_content[key] = value

            if key == 'user.token' and not content[key]:
                self.cookie = content[key]

    def get_config_content(self):
        try:
            with open(self.conf_file, 'r') as f:
                config_content = json.load(f)
        except json.decoder.JSONDecodeError:
            config_content = {}

        return config_content

    def check_config(self):
        res = self.get_config_content()
        if not res:
            init_config_dict = {
                'opendatalab_session': '',
            }
            result = init_config_dict
            with open(self.conf_file, 'w') as f:
                json.dump(init_config_dict, f, indent=4, sort_keys=True, separators=(',', ':'))
        else:
            result = res

        return result

    def update_config(self, content: dict) -> None:
        res = self.get_config_content()
        if res:
            self.set_content(content)
            with open(self.conf_file, 'w') as f:
                f.seek(0)
                json.dump(self._conf_content, f, indent=4, sort_keys=True, separators=(',', ':'))

    def clean_config(self):
        res = self.get_config_content()
        if not res:
            self.check_config()
        else:
            with open(self.conf_file, "w") as f:
                if 'user.token' in res.keys():
                    res['user.token'] = ''
                if 'user.email' in res.keys():
                    res['user.email'] = ''
                f.seek(0)
                json.dump(res, f, indent=4, sort_keys=True, separators=(',', ':'))

        return res


def exception_handler(func: _Callable) -> _Callable:
    """Decorator for CLI functions to catch custom exceptions.

    Arguments:
        func: The CLI function needs to be decorated.

    Returns:
        The CLI function with exception catching procedure.

    """

    @wraps(func)
    def wrapper(*args: Any, **kwargs: Any) -> Any:
        try:
            func(*args, **kwargs)
        except OpenDataLabError as err:
            if err.STATUS_CODE == 401:
                print(f"Error: authentication failure, please login!")
                pass
            elif err.STATUS_CODE == 403:
                print(f"Unable to access. Please visit the dataset homepage!")
                pass
            elif err.STATUS_CODE == 404:
                print(f"Data not exists!")
                pass
            elif err.STATUS_CODE == 412:
                print(f"Access with cdn error!")
                pass
            elif err.STATUS_CODE == 500:
                print(f"Internal server occurs!")
                pass
            else:
                print(f"Error occurs!!!")

            sys.exit(1)

    return wrapper  # type: ignore[return-value]


class ContextInfoNoLogin:
    """This class contains command context, the command no need to login"""

    def __init__(self):
        self.url = endpoint

    def get_client(self) -> Client:
        return Client(host=self.url, odl_cookie='')



"""
update dataset repository
"""
from rich import print as rprint

from openxlab.dataset.commands.utility import ContextInfo
from openxlab.dataset.exception import *


def visibility(dataset_repo: str, private: bool):
    """
    set dataset visibility.

    Example:
        openxlab.dataset.visibility(
            dataset_repo="username/dataset_repo_name",
            private=True
        )

    Parameters:
        @dataset_repo String The address of dataset repository.
        @private String The visibility permission of repository.
    """
    ctx = ContextInfo()
    client = ctx.get_client()
    permission = 'private' if private else 'public'

    # parse dataset_name
    parsed_ds_name = dataset_repo.replace("/", ",")
    # check dataset repository validation to set status as public
    if private:
        check_resp = client.get_api().check_public_validation(parsed_ds_name)
    resp = client.get_api().set_repo_permission(parsed_ds_name, private)

    rprint(f"Visibility: [blue]{dataset_repo}[/blue] now is {permission}.")



# odl_clientId = "kmz3bkwzlaa3wrq8pvwa"
# odl_clientId = "qja9jy5wnjyqwvylmeqw"
# uaa_url_prefix = "https://sso.openxlab.org.cn/gw/uaa-be"
# uaa_url_prefix = "https://sso.staging.openxlab.org.cn/gw/uaa-be"
# baseUrl: https://sso.staging.openxlab.org.cn/gw/uaa-be

# TODO toggle this when publishing
# production
# odl_clientId = "kmz3bkwzlaa3wrq8pvwa"
# staging
# odl_clientId= 'ypkl8bwo0eb5ao1b96no'


# endpoint = "http://staging.openxlab.org.cn"

# uaa_url_prefix = "https://sso.staging.openxlab.org.cn/gw/uaa-be"
# uaa_url_prefix = "https://sso.openxlab.org.cn/gw/uaa-be"

FILE_THRESHOLD = 30 * 1024 * 1024
# FILE_THRESHOLD = 100


# # staging
# odl_clientId = 'ypkl8bwo0eb5ao1b96no'
# endpoint = "https://staging.openxlab.org.cn"
# uaa_url_prefix = "https://sso.staging.openxlab.org.cn/gw/uaa-be"

# # prod
odl_clientId = "kmz3bkwzlaa3wrq8pvwa"
endpoint = "https://openxlab.org.cn"
uaa_url_prefix = "https://sso.openxlab.org.cn/gw/uaa-be"


computed_url = "/datasets/api/v2/"



from openxlab.model.handler.download_file import download as download, download_metafile_template as init, download_from_url as wget
from openxlab.model.handler.create_repository import create
from openxlab.model.handler.update_repository import visibility
from openxlab.model.handler.remove_repo_or_file import remove
from openxlab.model.handler.query_repo_model import list
from openxlab.model.handler.upload_file import upload
from openxlab.model.handler.model_inference import Inference
from openxlab.model.handler.model_inference import inference





""" 
get the list of dataset repository
"""
from rich import box
from rich.console import Console
from rich.table import Table

from openxlab.dataset.commands.utility import ContextInfoNoLogin
from openxlab.dataset.exception import *
from openxlab.dataset.utils import bytes2human


def query(dataset_repo: str):
    """
    List dataset repository resources.
    Note: if you are not log in, you can only get the list of public dataset repository.

    Example:
        openxlab.dataset.query(dataset_repo="username/dataset_repo_name")

    Parameters:
        @dataset_repo String The address of dataset repository.
    """
    ctx = ContextInfoNoLogin()
    client = ctx.get_client()

    parsed_ds_name = dataset_repo.replace("/", ",")
    data_dict = client.get_api().get_dataset_files(dataset_name=parsed_ds_name, needContent=True)
    # track
    client.get_api().track_query_dataset_files(dataset_name=parsed_ds_name)
    object_info_dict = {}
    total_size = 0
    for info in data_dict['list']:
        object_info_dict[info['path']] = bytes2human(info['size'])
        total_size += info['size']
    if len(object_info_dict) == 0:
        raise OdlAccessDeniedError()

    sorted_object_info_dict = dict(
        sorted(object_info_dict.items(), key=lambda x: x[0], reverse=True)
    )

    console = Console()
    table = Table(show_header=True, header_style='bold cyan', box=box.ASCII2)  # ROUNDED
    table.add_column("File Name", min_width=20, justify='left')
    table.add_column("Size", width=12, justify='left')

    print(f"Total Size: {bytes2human(total_size)}")
    for key, val in sorted_object_info_dict.items():
        table.add_row(key, val, end_section=True)
    console.print(table)

    return sorted_object_info_dict



"""
get model list of repository-cli
"""
from openxlab.types.command_type import *
from openxlab.model import list


class List(BaseCommand):
    """To view model information and model meta information under the model repository."""

    def get_name(self) -> str:
        return "list"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = 'openxlab model list [-h] -r MODEL_REPO [-i METAFILE] {help} ...\n' \
                       'Example:\n' \
                       '> openxlab model list --model-repo=username/model_repo_name'
        parser.add_argument('-r', '--model-repo', required=True,
                            help='The name of model repository.[required]')
        parser.add_argument('-i', '--metafile', type=bool, required=False,
                            help='View model meta information, default is Fasle.')

    def take_action(self, parsed_args: Namespace) -> int:
        list(parsed_args.model_repo, parsed_args.metafile)
        return 0



"""
模型推理功能
"""

import base64
import os
import re
import json

import requests

from openxlab.model.clients.modelapi_client import ModelApiClient
from openxlab.model.clients.openapi_client import OpenapiClient
from openxlab.model.common.constants import endpoint, token


class Inference:
    def __init__(self, model_repo):
        self.model_repo = model_repo
        # self.device = 'cpu' if device is None else 'gpu'

    def inference(self, input, **args):
        """
        model inference
        """
        try:
            # split params
            username, repository = _split_repo(self.model_repo)
            client = OpenapiClient(endpoint, token)

            model_repo_info = client.query_model_repo_info(username, repository)
            model_api_client = ModelApiClient(model_repo_info['inferUrl'], token)
            texts, files = _process_inputs(input)
            payload = {"files": files, "texts": texts}
            if args:
                payload["args"] = json.dumps(args)
            else:
                payload["args"] = None
            # print(f'payload ->{payload}')
            result = model_api_client.get_inference_result(payload)
            # print(f"inference result:{result}.")
            return result
        except ValueError as e:
            print(f"Error: {e}")
            return


def inference(model_repo, input, **args):
    model_infer = Inference(model_repo)
    return model_infer.inference(input, **args)


def _split_repo(model_repo) -> (str, str):
    """
    Split a full repository name into two separate strings: the username and the repository name.
    """
    # username/repository format check
    pattern = r'.+/.+'
    if not re.match(pattern, model_repo):
        raise ValueError("The input string must be in the format 'username/model_repo'")

    values = model_repo.split('/')
    return values[0], values[1]


def _process_inputs(inputs):
    texts = []
    files = []
    if isinstance(inputs, list):
        inputs = inputs
    else:
        inputs = [inputs]
    for input_str in inputs:
        if input_str.startswith('http'):
            # if input is a URL
            response = requests.get(input_str)
            input_data = response.content
            files.append(input_data)
            # input_data = base64.b64encode(response.content).decode('utf-8')
        elif os.path.isfile(input_str):
            # if input is a file path
            filename = os.path.basename(input_str)
            files.append(("files", (filename, open(input_str, "rb"))))
            # input_data = base64.b64encode(file_data).decode('utf-8')
        else:
            # if input is just a string
            # input_data = input_str
            texts.append(input_str)
    return texts, files





import json
import sys
from typing import Dict
from urllib.parse import quote

import requests
from rich import print as rprint

from openxlab.dataset.constants import computed_url
from openxlab.dataset.exception import *
from openxlab.dataset.utils import highlight_urls


class XlabDatasetAPI(object):
    """This class contains the interaction between client & odl serverend.

    This class is being instantiate from client.py
    Strongly recommend that openxlab provide a general method to handle contextinfo.

    """

    def __init__(self, host, cookie):
        self.host = host
        self.odl_cookie = cookie

    def get_dataset_files(self, dataset_name: str, payload: dict = None, needContent=False):
        header_dict = {"accept": "application/json"}
        data = {"recursive": True, "needContent": needContent}

        if payload:
            data.update(payload)

        resp = requests.get(
            url=f"{self.host}/datasets/api/v2/datasets/{dataset_name}/r/main",
            params=data,
            headers=header_dict,
            cookies=self.odl_cookie,
        )

        if resp.status_code != 200:
            rprint(f"{OpenDataLabError(resp.status_code, highlight_urls(resp.text))}")
            sys.exit(-1)
        result_dict = resp.json()['data']
        # Any other files besides readme.md and metafile.yaml?
        if not result_dict['hasMediaFile']:
            rprint(f"{OpenDataLabError(error_msg=highlight_urls(result_dict['toast']))}")
            sys.exit(-1)
        # no content
        if not result_dict['list']:
            print(f"{OpenDataLabError(404, 'get dataset list failed.')}")
            sys.exit(-1)
        return result_dict

    def download_check(self, dataset_id: str, path: str):
        """check file when download it for crawler"""
        resp = requests.get(
            url=f"{self.host}/datasets/api/v2/downloadCheck/{dataset_id}/main/{path}",
            headers={"accept": "application/json"},
            cookies=self.odl_cookie,
        )

        if resp.status_code != 200:
            rprint(f"{OpenDataLabError(resp.status_code, highlight_urls(resp.text))}")
            sys.exit(-1)
        return

    def get_dataset_download_urls(self, dataset_id: int, dataset_dict: Dict):
        resp = requests.get(
            url=f"{self.host}/datasets/resolve/{dataset_id}/main/{dataset_dict['name']}",
            headers={"accept": "application/json"},
            cookies=self.odl_cookie,
            allow_redirects=False,
        )
        if resp.status_code != 302:
            print(f"{OpenDataLabError(resp.status_code, resp.text)}")
            sys.exit(-1)
        return resp.headers['Location']

    def get_dataset_info(self, dataset_name: str):
        header_dict = {
            "accept": "application/json",
        }
        parsed_dataset_name = quote(dataset_name.replace("/", ","))
        resp = requests.get(
            url=f"{self.host}{computed_url}datasets/{parsed_dataset_name}",
            headers=header_dict,
            cookies=self.odl_cookie,
        )

        if resp.status_code != 200:
            print(f"{OpenDataLabError(resp.status_code, resp.text)}")
            sys.exit(-1)

        data = resp.json()['data']
        if data['id'] == 0:
            print(f"No dataset:{dataset_name}")
            sys.exit(-1)
        return data

    def pre_object_upload(self, dataset: str, branch: str, file_path: str, req: dict) -> dict:
        resp = requests.post(
            url=f"{self.host}{computed_url}preUpload/{dataset}/{branch}{file_path}",
            data=json.dumps(req),
            headers={
                "Content-Type": "application/json",
                "accept": "application/json",
            },
            cookies=self.odl_cookie,
        )
        if resp.status_code != 200:
            print(f"{OpenDataLabError(resp.status_code, resp.text)}")
            sys.exit(-1)
        resp_json = resp.json()
        data = resp_json['data']
        return data

    def post_object_upload(self, dataset: str, branch: str, file_path: str, req: dict) -> dict:
        # optimize
        if file_path.startswith('/'):
            file_path = file_path.split('/', 1)[-1]
        resp = requests.post(
            url=f"{self.host}{computed_url}postUpload/{dataset}/{branch}/{file_path}",
            data=json.dumps(req),
            headers={
                "Content-Type": "application/json",
                "accept": "application/json",
            },
            cookies=self.odl_cookie,
        )
        if resp.status_code != 200:
            print(f"{OpenDataLabError(resp.status_code, resp.text)}")
            sys.exit(-1)
        resp_json = resp.json()
        data = resp_json["data"]
        return data

    def create_dataset(self, req: dict):
        resp = requests.post(
            url=f"{self.host}/datasets/api/v2/datasets",
            data=json.dumps(req),
            headers={"Content-Type": "application/json", "accept": "application/json"},
            cookies=self.odl_cookie,
        )
        if resp.status_code != 200:
            print(f"While creating {req['name']} encounter error: {OpenDataLabError(resp.text)}")
            sys.exit(-1)
        resp_json = resp.json()
        data = resp_json['data']
        return data

    def commit_dataset(self, req: list):
        dataset_name = req[0]
        parsed_dataset_name = quote(dataset_name.replace("/", ","))

        resp = requests.post(
            url=f"{self.host}/datasets/api/v2/datasets/{parsed_dataset_name}/commit",
            data=json.dumps(req[1]),
            headers={"Content-Type": "application/json", "accept": "application/json"},
            cookies=self.odl_cookie,
        )

        if resp.status_code != 200:
            print(f"While committing {req[0]} encounter error: {OpenDataLabError(resp.text)}")
            sys.exit(-1)
        return

    def delete_repo(self, dataset_repo_name: str):
        """delete repo api"""
        resp = requests.delete(
            url=f"{self.host}{computed_url}datasets/{dataset_repo_name}",
            cookies=self.odl_cookie,
        )

        if resp.status_code != 200:
            dataset_repo_name = dataset_repo_name.replace(",", "/")
            print(
                f"While deleting {dataset_repo_name} encounter error: {OpenDataLabError(resp.text)}"
            )
            sys.exit(-1)
        return

    def set_repo_permission(self, dataset_repo_name: str, private: bool):
        """
        change repo permission
        @private: true is private, false is public
        """
        permission = "private" if private else "public"
        header_dict = {"accept": "application/json"}
        resp = requests.post(
            url=f"{self.host}{computed_url}datasets/{dataset_repo_name}/actions/changeState?state={permission}",
            headers={"Content-Type": "application/json", "accept": "application/json"},
            cookies=self.odl_cookie,
        )

        if resp.status_code != 200:
            ds_name = dataset_repo_name.replace(",", "/")
            print(
                f"while change {ds_name} visibility encounter error:{OpenDataLabError(resp.text)}"
            )
            sys.exit(-1)
        return

    def track_query_dataset_files(self, dataset_name: str):
        """track the operation of getting dataset files"""
        header_dict = {"Content-Type": "application/json", "accept": "application/json"}

        resp = requests.post(
            url=f"{self.host}/datasets/api/v2/track/datasets/cli/ls/{dataset_name}",
            headers=header_dict,
            cookies=self.odl_cookie,
        )

        if resp.status_code != 200:
            print(f"{OpenDataLabError(resp.status_code, resp.text)}")
            sys.exit(-1)
        return

    def track_download_dataset_files(self, dataset_name: str, file_path: str):
        """track the operation of downloading dataset files"""
        header_dict = {"Content-Type": "application/json", "accept": "application/json"}
        body = {"path": file_path}
        resp = requests.post(
            url=f"{self.host}/datasets/api/v2/track/datasets/cli/download/{dataset_name}",
            headers=header_dict,
            cookies=self.odl_cookie,
            data=body,
        )

        if resp.status_code != 200:
            print(f"{OpenDataLabError(resp.status_code, resp.text)}")
            sys.exit(-1)
        return

    def check_public_validation(
        self,
        dataset_name: str,
    ):
        """check dataset repository can be set public or not"""
        resp = requests.post(
            url=f"{self.host}/datasets/api/v2/datasets/{dataset_name}/actions/validate",
            headers={"Content-Type": "application/json", "accept": "application/json"},
            cookies=self.odl_cookie,
        )
        if resp.status_code != 200:
            print(
                f"While check public validation of {dataset_name} encounter error: {OpenDataLabError(resp.text)}"
            )
            sys.exit(-1)
        resp_json = resp.json()
        data = resp_json['data']
        return data



import requests

from openxlab.xlab.handler.user_token import get_jwt
from openxlab.utils.id_util import generate_unique_id
import json
import base64
import os



class ModelApiClient(object):
    def __init__(self, endpoint, token):
        self.token = token
        self.endpoint = endpoint

    def get_inference_result(self, payload):
        """
        get inference result
        """
        result = self.http_post_response_dto(payload)
        return result

    def http_post_response_dto(self, payload):
        headers = self.http_common_header()
        data = {'texts': payload['texts']}
        if payload['args'] is not None:
            data['args'] = payload['args']
        print(f'payload:{payload}')
        response = requests.post(self.endpoint, files=payload["files"], data=data,
                                 headers=headers)
        response.raise_for_status()
        result = Result(payload, response.content, response.headers['Content-Type'])
        return result

    def http_common_header(self):
        try:
            jwt = get_jwt()
        except ValueError as e:
            print(f"warning: {e}")
            return
        header_dict = {
            "Authorization": jwt
        }
        return header_dict


class Result(object):
    def __init__(self, payload, original, content_type):
        self.payload = payload
        self.original = original
        self.content_type = content_type

    def tojson(self):
        return json.loads(self.original)

    @property
    def predictions(self):
        if 'application/json' not in self.content_type:
            return None
        data = json.loads(self.original)
        if type(data) == dict:
            return data['predictions']
        else:
            for item in data:
                if 'visualization' in item:
                    del item['visualization']
        return data

    @property
    def visualization(self):
        data = json.loads(self.original)
        if 'application/json' not in self.content_type:
            return None
        if type(data) == dict:
            return data['visualization']
        else:
            visualization_list = [item["visualization"] for item in data]
            return visualization_list

    def save_base64_images(self, output_dir=None):
        output_file_list = []
        visualization = self.visualization
        if not visualization:
            return visualization

        input_files = self.payload['files']
        if len(input_files) != len(visualization):
            raise ValueError('input files length is not match output visualization length')

        if output_dir is not None:
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
        else:
            output_dir = os.getcwd()

        for i, item in enumerate(visualization):
            image_data_arr = item.split(',')
            if len(image_data_arr) == 2:
                image_data = image_data_arr[1]
            else:
                image_data = image_data_arr[0]
            base_name = os.path.basename(input_files[i][1][0])
            file_name_without_extension, extension = os.path.splitext(base_name)
            filename = f"{file_name_without_extension}_visual_{generate_unique_id(6)}{extension}"
            file_path = os.path.join(output_dir, filename)

            image_bytes = base64.b64decode(image_data)

            with open(file_path, "wb") as file:
                file.write(image_bytes)
            output_file_list.append(file_path)
        return output_file_list





"""
model inference-cli
"""
from openxlab.types.command_type import *
from openxlab.model.handler.model_inference import Inference


class Inference(BaseCommand):
    """inference"""

    def get_name(self) -> str:
        return "inference"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument('-r', '--model-repo', required=True,
                            help='model repository address. format:username/repository.')
        parser.add_argument('-i', '--input', type=str, nargs="+", required=True,
                            help='input content, text.')
        parser.add_argument('-n', '--model-name', required=True,
                            help='model name.')
        parser.add_argument('-d', '--device', required=False,
                            help='device name.')

    def take_action(self, parsed_args: Namespace) -> int:
        inferencer = Inference(parsed_args.model_repo, parsed_args.model_name, parsed_args.device)
        inferencer.inference(parsed_args.input)
        return 0





import os

from openxlab.xlab.handler.user_config import UserConfig, get_config_path
from openxlab.xlab.handler.user_token import get_token_via_api
import logging

logger = logging.getLogger("openxlab")


def login(ak, sk, re_login=False, relogin=False):
    if os.path.exists(get_config_path()) and re_login is False and relogin is False:
        logger.warning("AK and SK have been configured. You can set relogin as true to force a relogin.")
    get_token_via_api(ak, sk)
    user_config = UserConfig(ak, sk)
    user_config.store_to_local()



import os.path
import time
import json

from openxlab.config import const


class LocalCache:
    def __init__(self, file_path):
        self.file_path = file_path
        self.cache = self.load_cache()

    def load_cache(self):
        try:
            with open(self.file_path, 'r') as f:
                return json.load(f)
        except FileNotFoundError:
            return {}

    def save_cache(self):
        with open(self.file_path, 'w') as f:
            json.dump(self.cache, f)

    def set(self, key, value, expire=0):
        if expire > 0:
            expire += int(time.time())
        self.cache[key] = {'expire': expire, 'data': value}
        self.save_cache()

    def get(self, key):
        if key not in self.cache:
            return None
        if 0 < self.cache[key]['expire'] < int(time.time()):
            del self.cache[key]
            self.save_cache()
            return None
        return self.cache[key]['data']

    def delete(self, key):
        if key in self.cache:
            del self.cache[key]
            self.save_cache()


cache = LocalCache(os.path.join(const.DEFAULT_CONFIG_DIR, 'cache.json'))



import requests

from openxlab.model.common.constants import paths
from openxlab.model.common.response_dto import ReturnDto


class UploadServiceClient(object):
    def __init__(self, endpoint, token):
        self.endpoint = endpoint
        self.token = token

    def get_upload_signature(self, repository_id, uid, model, model_info):
        payload = {"tag": "file", "key": "model-center", "type": 0, "fileType": "file/pth",
                   "fileName": model_info['objectName'],
                   "data": {"repositoryId": repository_id, "modelId": model_info['modelId'],
                            "modelName": model_info['modelName'], "weightName": model_info['weightName'],
                            "uid": uid}}
        response_dto = self.http_post_response_dto("", payload)
        if response_dto.msg_code != '10000':
            raise ValueError(f"Get upload signature error:{response_dto.msg}, traceId:{response_dto.trace_id}")
        if response_dto.data['msgCode'] != '10000':
            raise ValueError(f"Get upload signature error:{response_dto.data['msg']}, "
                             f"traceId:{response_dto.data['traceId']}")
        return response_dto.data['data']

    def http_post_response_dto(self, path, payload):
        headers = self.http_common_header()
        print(f"headers:{headers}, body:{payload}")
        response = requests.post(f"{self.endpoint}{path}", json=payload, headers=headers)
        response.raise_for_status()
        response_dict = response.json()
        response_dto = ReturnDto.from_camel_case(response_dict)
        return response_dto

    def http_common_header(self):
        header_dict = {
            "Content-Type": "application/json",
            "accept": "application/json",
            "id": "597091",
            "Authorization": f"Bearer {self.token}"
        }
        return header_dict





""" 
get dataset repository totally
"""

import os

from rich import print as rprint
from tqdm import tqdm

from openxlab.dataset.commands.utility import ContextInfo
from openxlab.dataset.constants import FILE_THRESHOLD
from openxlab.dataset.io import downloader
from openxlab.types.command_type import *


def get(dataset_repo: str, target_path=""):
    """
    Get the dataset repository.

    Example:
        openxlab.dataset.get(
            dataset_repo="username/dataset_repo_name",
            target_path="/path/to/local/folder"
        )

    Parameters:
        @dataset_repo String The address of dataset repository.
        @target_path String The target local path to save the dataset repository.
    """
    if not target_path:
        target_path = os.getcwd()
    # else:
    #     destination_path = os.path.normpath(destination_path)

    ctx = ContextInfo()
    client = ctx.get_client()

    # parse dataset_name
    parsed_ds_name = dataset_repo.replace("/", ",")
    parsed_save_path = dataset_repo.replace("/", "___")

    data_dict = client.get_api().get_dataset_files(dataset_name=parsed_ds_name)
    info_dataset_id = data_dict['list'][0]['dataset_id']

    object_info_list = []
    for info in data_dict['list']:
        curr_dict = {}
        curr_dict['size'] = info['size']
        curr_dict['name'] = info['path'][1:]
        object_info_list.append(curr_dict)

    if object_info_list:
        # download check for crawler with one file
        download_check_path = object_info_list[0]['name']
        # download check
        client.get_api().download_check(dataset_id=info_dataset_id, path=download_check_path)

    with tqdm(total=len(object_info_list)) as pbar:
        for idx in range(len(object_info_list)):
            # exist already
            rprint(f" Downloading No.{idx+1} of total {len(object_info_list)} files")
            if os.path.exists(
                (os.path.join(target_path, parsed_save_path, object_info_list[idx]['name']))
            ):
                rprint(f"target already exists, jumping to next !")
                pbar.update(1)
                continue

            # big file download
            if object_info_list[idx]['size'] > FILE_THRESHOLD:
                download_url = client.get_api().get_dataset_download_urls(
                    info_dataset_id, object_info_list[idx]
                )
                downloader.BigFileDownloader(
                    url=download_url,
                    filename=object_info_list[idx]['name'],
                    download_dir=os.path.join(target_path, parsed_save_path),
                    file_size=object_info_list[idx]['size'],
                    blocks_num=8,
                ).start()
                pbar.update(1)
            # small file download
            else:
                download_url = client.get_api().get_dataset_download_urls(
                    info_dataset_id, object_info_list[idx]
                )
                downloader.SmallFileDownload(
                    url=download_url,
                    filename=object_info_list[idx]['name'],
                    download_dir=os.path.join(target_path, parsed_save_path),
                )._single_thread_download()
                pbar.update(1)
    client.get_api().track_download_dataset_files(dataset_name=parsed_ds_name, file_path="")
    rprint(f"\nDownload Completed.")



"""
setting dataset repository visibility-cli
"""

from argparse import ArgumentParser
from argparse import Namespace

from openxlab.dataset.handler.visible_dataset_repository import visibility
from openxlab.dataset.utils import str2bool
from openxlab.types.command_type import *


class Visibility(BaseCommand):
    """Set dataset repository visibility to public or private"""

    def get_name(self):
        return "visibility"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = (
            'openxlab dataset visibility [OPTIONS]\n\n'
            "set dataset visibility.\n\n"
            'Example:\n'
            '> openxlab dataset visibility --dataset-repo \"username/dataset-repo-name\" --private True'
        )
        parser.add_argument(
            "-r",
            "--dataset-repo",
            required=True,
            help='The address of dataset repository. format: username/dataset-repo-name.[required]',
        )
        parser.add_argument(
            "-p",
            "--private",
            type=str2bool,
            # default=True,
            required=True,
            help='The visibility permission of repository.[required]',
        )

    def take_action(self, parsed_args: Namespace) -> int:
        visibility(parsed_args.dataset_repo, parsed_args.private)

        return 0



import re


class ReturnDto:
    def __init__(self, trace_id=None, msg_code=None, msg=None, data=None, total=None, success=None):
        self.trace_id = trace_id
        self.msg_code = msg_code
        self.msg = msg
        self.data = data
        self.total = total
        self.success = success

    # 将驼峰型属性名转换为下划线型属性名
    @classmethod
    def from_camel_case(cls, camel_dict):
        snake_dict = {}
        for key, value in camel_dict.items():
            snake_key = re.sub(r'(?<!^)(?=[A-Z])', '_', key).lower()
            snake_dict[snake_key] = value
        return cls(**snake_dict)


class DownloadUrlDto:
    def __int__(self, url=None, updating=None, updating_readme=None):
        self.url = url
        self.updating = updating
        self.updating_readme = updating_readme



"""
setting model repository visibility-cli
"""
from openxlab.types.command_type import *
from openxlab.model import visibility


class Visibility(BaseCommand):
    """visibility"""

    def get_name(self) -> str:
        return "visibility"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument('-r', '--model-repo', required=True,
                            help='model repository address. format:username/repository.')
        parser.add_argument('-prt', '--private', type=bool, default=False, required=False,
                            help='set repository visibility.')

    def take_action(self, parsed_args: Namespace) -> int:
        private = False
        visibility(parsed_args.model_repo, private)
        return 0



import os


def print_directory(path='/', hide_file=False, print_depth=10):
    if print_depth < 1:
        return
    print(f"🌳 目录树: {path}")

    # 用于缩进目录层级的计数器
    indent = 0

    # 递归打印目录树的辅助函数
    def print_tree(current_path, indent):
        if indent >= print_depth:
            return

        # 获取当前路径下的所有文件和文件夹
        for item in os.listdir(current_path):
            # 排除隐藏文件和文件夹（如果hide_file为False）
            if not hide_file and item.startswith('.'):
                continue

            # 构建文件或文件夹的完整路径
            item_path = os.path.join(current_path, item)

            # 打印文件或文件夹名称，使用缩进表示层级
            print('  ' * indent + '|-- ' + item)

            # 递归打印子目录的目录树
            if os.path.isdir(item_path):
                print_tree(item_path, indent + 1)

    print_tree(path, indent)



import base64
import hmac

from openxlab.utils.time_util import *
from openxlab.xlab.clients.auth_client import AuthClient
from openxlab.xlab.handler.user_config import *


AUTH_CLIENT = AuthClient("https://openapi.openxlab.org.cn/api/v1/sso-be/api/v1/open/")
# AUTH_CLIENT = AuthClient("https://staging.openxlab.org.cn/api/v1/sso-be/api/v1/open/")


def calculate_d(sk, nonce, algorithm):
    if len(sk) == 0 or len(nonce) == 0 or len(algorithm) == 0:
        raise ValueError("sk, nonce and algorith must not be empty")
    try:
        hmac_key = bytearray(sk.encode('utf-8'))
        hmac_obj = hmac.new(
            hmac_key,
            nonce.encode('utf-8'),
            algorithm[4:] if algorithm.startswith("Hmac") else algorithm,
        )
        raw_hmac = hmac_obj.digest()
        return base64.b64encode(raw_hmac).decode('utf-8')
    except KeyError as e:
        raise ValueError("Unsupported hash type: %s" % algorithm) from e
    except Exception as e:
        raise ValueError("Error signing nonce: %s" % str(e)) from e


def get_jwt(ak=None, sk=None):
    return get_token(ak, sk).jwt


def get_token(ak=None, sk=None):
    local_user_token = get_token_from_local()
    if local_user_token is None:
        return get_token_via_api(ak, sk)
    user_token_expiration_datetime = get_datetime_from_formatted_str(local_user_token.expiration)
    now = get_current_time()
    if user_token_expiration_datetime <= now:
        return refresh_token(ak, sk)
    return local_user_token


def get_token_from_local():
    if not os.path.exists(get_token_path()):
        return None
    token_json = get_file_content(get_token_path())
    token_dict = json.loads(token_json)
    return UserToken(**token_dict)


def get_token_via_api(ak=None, sk=None):
    user_config = get_config(ak, sk)
    if user_config is None:
        raise ValueError(
            "Local config must not be empty before get token via api. "
            "Please use the 'openxlab config' command to set the config"
        )
    nonce, algorithm = AUTH_CLIENT.auth(user_config.ak)
    d = calculate_d(user_config.sk, nonce, algorithm)
    jwt_dict = AUTH_CLIENT.get_jwt(user_config.ak, d)
    jwt_dict['refresh_time'] = get_current_formatted_time()
    user_token = UserToken(**jwt_dict)
    # user_token.store_to_local()
    return user_token


def refresh_token(ak=None, sk=None):
    user_config = get_config(ak, sk)
    if user_config is None:
        raise ValueError(
            "Local config must not be empty before refresh token. "
            "Please use the 'openxlab config' command to set the config"
        )
    local_user_token = get_token_from_local()
    if local_user_token is None:
        raise ValueError("local token must not be empty before refresh token")
    refresh_expiration_datetime = get_datetime_from_formatted_str(
        local_user_token.refresh_expiration
    )
    now = get_current_time()
    if refresh_expiration_datetime <= now:
        return get_token_via_api(ak, sk)
    refresh_jwt_dict = AUTH_CLIENT.refresh_jwt(user_config.ak, local_user_token.refresh_token)
    refresh_jwt_dict['refresh_time'] = get_current_formatted_time()
    user_token = UserToken(**refresh_jwt_dict)
    # user_token.store_to_local()
    return user_token


class UserToken(object):
    def __init__(self, jwt, expiration, sso_uid, refresh_time, refresh_token, refresh_expiration):
        self.jwt = jwt
        self.expiration = expiration
        self.sso_uid = sso_uid
        self.refresh_time = refresh_time
        self.refresh_token = refresh_token
        self.refresh_expiration = refresh_expiration

    def to_json(self):
        return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)

    def store_to_local(self):
        if not os.path.exists(get_config_dir()):
            os.makedirs(get_config_dir(), mode=0o700)
        token_json = self.to_json()
        with open(get_token_path(), mode="w", encoding='utf-8') as f:
            f.write(token_json)



"""
download model file|meta file|log file|readme file-cli
"""
from openxlab.types.command_type import *
from openxlab.model import wget


class Wget(BaseCommand):
    """Download model weight files."""

    def get_name(self) -> str:
        return "wget"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = "openxlab model wget [-h] -r MODEL_REPO -f FILE [-p PATH] [-o] {help} ...\n" \
                       'Example:\n' \
                       '> openxlab model wget --url={url}' \
                       '  --output=/path/to/local/folder/\n' \
                       '  --overwrite=True'
        parser.add_argument('-u', '--url', required=True,
                            help='model remote download url.')
        parser.add_argument('-op', '--output', required=False,
                            help='setting download path.')
        parser.add_argument('-o', '--overwrite', default=False, type=bool,
                            help='force overwriting local files.')

    def take_action(self, parsed_args: Namespace) -> int:
        wget(parsed_args.url, path=parsed_args.output, overwrite=parsed_args.overwrite)
        return 0



from openxlab.config import version as config_version
from openxlab.types.command_type import *


def help():
    print("help")


class Upload(BaseCommand):
    """upload"""

    def get_name(self) -> str:
        return "upload"

    def add_arguments(self, parser: ArgumentParser) -> None:
        pass

    def take_action(self, parsed_args: Namespace) -> int:
        print("upload something")
        return 0


class Demo(BaseCommand):
    """demo"""

    sub_command_list = [Upload]

    def get_name(self) -> str:
        return "demo"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "--directions",
            type=str,
            nargs="+",
            choices=("minimize", "maximize"),
            help=(
                "Directions of optimization, if there are multiple objectives."
                " This argument is deprecated. Please create a study in advance."
            ),
        )

    # def take_action(self, parsed_args: Namespace) -> int:
    #     help()
    #     return 0



import requests

from openxlab.model.common.constants import paths
from openxlab.model.common.response_dto import ReturnDto
from openxlab.model.common.meta_file_util import get_meta_payload, get_filename_from_url
import os
from openxlab.xlab.handler.user_token import get_jwt


class OpenapiClient(object):
    def __init__(self, endpoint, token=None):
        self.endpoint = endpoint
        self.token = token

    # def get_dataset_files(self, dataset_name:str, prefix:str):
    def get_download_url(self, username, repository, model_name, filepath, ignore):
        """
        get file(model file|meta file|log file|readme file) download url
        """
        if isinstance(ignore, str):
            ignore = [ignore]
        client_from = os.environ.get('CLIENT_FROM', '0')
        payload = {"userName": username, "repositoryName": repository, "modelNames": model_name, "filePaths": filepath,
                   "clientFrom": client_from, "ignore": ignore}
        path = paths['file_download_path']
        response_dto = self.http_post_response_dto(path, payload)
        if response_dto.msg_code != '10000':
            raise ValueError(f"Get download url error:{response_dto.msg}, traceId:{response_dto.trace_id}")
        if response_dto.data['msgCode'] != '10000':
            raise ValueError(f"Get download url error:{response_dto.data['msg']}, "
                             f"traceId:{response_dto.data['traceId']}")
        return response_dto.data['data']['modelNames'], response_dto.data['data']['filePaths']

    def get_metafile_template_download_url(self, file=None, all=False):
        """
        get metafile template download url
        """
        payload = {'all': all}
        path = paths['meta_file_template_download_path']
        response_dto = self.http_post_response_dto(path, payload, auth=False)
        if response_dto.msg_code != '10000':
            raise ValueError(f"Get download url error:{response_dto.msg}, traceId:{response_dto.trace_id}")
        if response_dto.data['msgCode'] != '10000':
            raise ValueError(f"Get download url error:{response_dto.data['msg']}, "
                             f"traceId:{response_dto.data['traceId']}")
        return response_dto.data['data']['url']

    def create_repository(self, repository: str, private: bool, meta_data: dict):
        """
        create repository
        """
        payload = get_meta_payload(repository, private, meta_data)
        path = paths['create_repository_path']
        response_dto = self.http_post_response_dto(path, payload)
        if response_dto.msg_code != '10000':
            raise ValueError(f"Create repository error:{response_dto.msg}, traceId:{response_dto.trace_id}")
        if response_dto.data['msgCode'] != '10000':
            raise ValueError(f"Create repository error:{response_dto.data['msg']}, "
                             f"traceId:{response_dto.data['traceId']}")
        return response_dto.data

    def update_repository(self, repository: str, private: bool):
        """
        update repository
        """
        public_status = 0 if private else 1
        payload = {"repoName": repository, "publicStatus": public_status}
        path = paths['update_repository_path']
        response_dto = self.http_post_response_dto(path, payload)
        if response_dto.msg_code != '10000':
            raise ValueError(f"Update repository error:{response_dto.msg}, traceId:{response_dto.trace_id}")
        if response_dto.data['msgCode'] != '10000':
            raise ValueError(f"Update repository error:{response_dto.data['msg']}, "
                             f"traceId:{response_dto.data['traceId']}")
        return response_dto.data

    def remove_repository(self, repository):
        """
        remove repository
        """
        payload = {"repoName": repository}
        path = paths['delete_repository_path']
        print(f'payload:{payload}')
        response_dto = self.http_post_response_dto(path, payload)
        if response_dto.msg_code != '10000':
            raise ValueError(f"Delete repository error:{response_dto.msg}, traceId:{response_dto.trace_id}")
        if response_dto.data['msgCode'] != '10000':
            raise ValueError(f"Delete repository error:{response_dto.data['msg']}, "
                             f"traceId:{response_dto.data['traceId']}")
        return response_dto.data

    def query_models(self, repository, metafile=None):
        """
        query models
        """
        payload = {"repoName": repository}
        path = paths['query_models_path']
        response_dto = self.http_post_response_dto(path, payload)
        if response_dto.msg_code != '10000':
            raise ValueError(f"Query models error:{response_dto.msg}, traceId:{response_dto.trace_id}")
        if response_dto.data['msgCode'] != '10000':
            raise ValueError(f"Query models error:{response_dto.data['msg']}, "
                             f"traceId:{response_dto.data['traceId']}")
        return response_dto.data['data']

    def update_upload_status(self, repository, file_names, new_models=None, update_models=None, upload_status=None):
        """
        update upload status
        """
        _new_models = None
        _update_models = None
        if new_models is not None:
            _new_models = [{"name": m['Name'], "weightName": get_filename_from_url(m['Weights']),
                            "evaluations": [{"task": e['Task'], "dataset": e['Dataset']} for e in m['Results']]} for m
                           in new_models]
        if update_models is not None:
            _update_models = [{"name": m['Name'], "weightName": get_filename_from_url(m['Weights'])} for m
                              in update_models]

        payload = {"repoName": repository, "fileNames": file_names, "uploadStatus": upload_status,
                   "models": _new_models, "updateModels": _update_models}
        path = paths['update_upload_status_path']
        response_dto = self.http_post_response_dto(path, payload)
        if response_dto.msg_code != '10000':
            raise ValueError(f"Update upload status error:{response_dto.msg}, traceId:{response_dto.trace_id}")
        if response_dto.data['msgCode'] != '10000':
            raise ValueError(f"Update upload status error:{response_dto.data['msg']}, "
                             f"traceId:{response_dto.data['traceId']}")
        return response_dto.data['data']

    def get_upload_signature(self, repository_id, uid, model, model_info):
        payload = {"tag": "file", "key": "model-center", "type": 0, "fileType": "file/pth",
                   "fileName": model_info['objectName'],
                   "data": {"repositoryId": repository_id, "modelId": model_info['modelId'],
                            "modelName": model_info['modelName'], "weightName": model_info['weightName'],
                            "uid": uid}}
        path = paths['get_upload_signature']
        response_dto = self.http_post_response_dto(path, payload)
        if response_dto.msg_code != '10000':
            raise ValueError(f"Get upload signature error:{response_dto.msg}, traceId:{response_dto.trace_id}")
        if response_dto.data['msgCode'] != '10000':
            raise ValueError(f"Get upload signature error:{response_dto.data['msg']}, "
                             f"traceId:{response_dto.data['traceId']}")
        return response_dto.data['data']

    def query_model_repo_info(self, username: str, repository: str):
        """
        query model repo info
        """
        payload = {"userName": username, "repoName": repository}
        path = paths['query_model_repo_info_path']
        response_dto = self.http_post_response_dto(path, payload)
        if response_dto.msg_code != '10000':
            raise ValueError(f"Get model repo info error:{response_dto.msg}, traceId:{response_dto.trace_id}")
        if response_dto.data['msgCode'] != '10000':
            raise ValueError(f"Get model repo info error:{response_dto.data['msg']}, "
                             f"traceId:{response_dto.data['traceId']}")
        return response_dto.data['data']

    def bury_data_upload(self, payload):
        """
        query model repo info
        """
        path = paths['bury_upload']
        response_dto = self.http_post_response_dto(path, payload, auth=False)
        if response_dto.msg_code != '10000':
            print(f"upload bury data error:{response_dto.msg}, traceId:{response_dto.trace_id}")
        if response_dto.data['msgCode'] != '10000':
            print(f"upload bury data error:{response_dto.data['msg']}, traceId:{response_dto.data['traceId']}")

    def http_post_response_dto(self, path, payload, auth=True):
        headers = self.http_common_header(auth)
        # print(f"headers:{headers}, body:{payload}")
        response = requests.post(f"{self.endpoint}{path}", json=payload, headers=headers)
        response.raise_for_status()
        response_dict = response.json()
        response_dto = ReturnDto.from_camel_case(response_dict)
        return response_dto

    def http_common_header(self, auth=True):
        header_dict = {
            "Content-Type": "application/json",
            "accept": "application/json"
        }
        if auth:
            try:
                jwt = get_jwt()
                header_dict['Authorization'] = jwt
            except ValueError as e:
                print(f"warning: {e}")
        return header_dict



"""
upload or update model file | log file | metafile
"""
import os
import re

from openxlab.model.clients.openapi_client import OpenapiClient
from openxlab.model.clients.oss_client import OssClient
from openxlab.model.common.bury import bury_data
from openxlab.model.common.constants import endpoint, token, oss_endpoint
from openxlab.model.common.meta_file_util import MetafileParser


@bury_data("upload_file")
def upload(model_repo, file_type='metafile', source=None, target=None, domain='bucket') -> None:
    """
    upload model file | readme file | logfile | meta file
    model_repo: model repo full name
    file_type: upload method.
    source: metafile location

    """
    try:
        # split params
        username, repository = _split_repo(model_repo)
        client = OpenapiClient(endpoint, token)
        """
        cli逻辑：
        metafile-根据metafile全部覆盖上传
        other-指定具体文件上传
        web端逻辑如下：
        1. 查询哪些待上传
        2. 更新上传状态-开始上传
        2. 遍历获取signature
        3. 上传oss
        4. 更新上传状态-上传完成
        """
        if file_type == 'metafile':
            remote_model_list = client.query_models(repository)['models']

            meta_parser = MetafileParser(source)
            meta_data = meta_parser.parse_and_validate()
            meta_file_model_list = meta_data['Models']
            upload_model_list(client, repository, meta_file_model_list, remote_model_list, domain)
        elif file_type == 'other':
            print('other way is designing')
        else:
            raise ValueError("file_type only support metafile/other")
        print(f"file upload successfully.")
    except ValueError as e:
        print(f"Error: {e}")
        return


def _split_repo(model_repo) -> (str, str):
    """
    Split a full repository name into two separate strings: the username and the repository name.
    """
    # username/repository format check
    pattern = r'.+/.+'
    if not re.match(pattern, model_repo):
        raise ValueError("The input string must be in the format 'username/model_repo'")

    values = model_repo.split('/')
    return values[0], values[1]


def upload_model_list(client: OpenapiClient, repository, meta_file_model_list, remote_model_list, domain):
    # 筛选出新增的models
    new_models = [m_meta for m_meta in meta_file_model_list
                  if m_meta['Name'] not in [m_remote['name'] for m_remote in remote_model_list]]

    update_models = []
    file_names = []
    for m_meta in meta_file_model_list:
        for m_remote in remote_model_list:
            if m_meta['Name'] == m_remote['name'] and os.path.basename(m_meta['Weights']) != m_remote['weightName']:
                file_names.append(m_remote['weightName'])
            else:
                file_names.append(os.path.basename(m_meta['Weights']))
            update_models.append(m_meta)

    models_data = client.update_upload_status(repository, file_names, new_models, update_models, 1)
    model_info_list = models_data['objects']
    for model in meta_file_model_list:
        model_info_filter_list = list(filter(lambda m: m['modelName'] == model['Name'], model_info_list))
        signature = client.get_upload_signature(models_data['repositoryId'], models_data['uid'], model,
                                                model_info_filter_list[0])
        oss_client = OssClient(oss_endpoint, signature['accessKeyId'], signature['accessKeySecret'],
                               signature['stsToken'],
                               signature['bucket'])
        # oss_client.upload_to_oss(model['Weights'], signature['object'], signature['callback'], signature['callbackVar'])
        oss_client.multipart_resume_upload(model['Weights'], signature['object'], signature['callback'], signature['callbackVar'], domain=domain)
        client.update_upload_status(repository, [os.path.basename(model['Weights'])], upload_status=2)



import json

from openxlab.config import const
from openxlab.utils.env_util import *
from openxlab.utils.file import *


def get_config_path() -> str:
    return os.path.join(get_config_dir(), get_config_file_name())


def get_token_path() -> str:
    return os.path.join(get_config_dir(), get_token_file_name())


def get_dataset_path() -> str:
    return os.path.join(get_config_dir(), get_dataset_file_name())


def get_config_dir() -> str:
    return const.DEFAULT_CONFIG_DIR


def get_token_file_name() -> str:
    return const.DEFAULT_CLI_TOKEN_FILE_NAME


def get_config_file_name() -> str:
    return const.DEFAULT_CLI_CONFIG_FILE_NAME


def get_dataset_file_name() -> str:
    return const.DEFAULT_CLI_DATASET_FILE_NAME


def get_config(ak=None, sk=None, need_force=False):
    """@need_force: utilized to check user's ak, if not, ask user to login"""
    if ak is not None and sk is not None:
        return UserConfig(ak, sk)
    if not os.path.exists(get_config_path()):
        ak_env_value = get_env(const.AK_ENV_NAME)
        sk_env_value = get_env(const.SK_ENV_NAME)
        if ak_env_value is not None and sk_env_value is not None:
            return UserConfig(ak_env_value, sk_env_value)
        # need login
        if need_force is True and not ak_env_value:
            raise Exception(
                "Please login openxlab and config ak/sk, try \"openxlab login\" and refer to https://openxlab.org.cn/docs/developers/鉴权管理.html"
            )
        return None
    config_json = get_file_content(get_config_path())
    config_dict = json.loads(config_json)
    return UserConfig(config_dict['ak'], config_dict['sk'])


def clear_dataset_json():
    """clear the content in dataset.json file"""
    try:
        with open(get_dataset_path(), 'w') as f:
            f.truncate(0)
    except Exception as e:
        raise (e)


class UserConfig(object):
    def __init__(self, ak, sk):
        if ak is None or sk is None:
            raise ValueError("ak and sk must not be empty")
        self.ak = ak
        self.sk = sk

    def to_json(self):
        return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)

    def store_to_local(self):
        if not os.path.exists(get_config_dir()):
            os.makedirs(get_config_dir(), mode=0o700)
        config_json = self.to_json()
        set_env(const.AK_ENV_NAME, self.ak)
        set_env(const.SK_ENV_NAME, self.sk)
        with open(get_config_path(), mode="w", encoding='utf-8') as f:
            f.write(config_json)



import sys
import time

from openxlab.model.clients.openapi_client import OpenapiClient
from openxlab.model.common.constants import endpoint
from openxlab.xlab.handler.user_token import get_token
from openxlab.model.common.thread_pool import bury_thread_pool


def bury_data_upload_task(_event_name_: str, client: OpenapiClient):
    payload = {'serviceName': 'model-center', 'eventName': _event_name_,
               'eventTimestamp': int(round(time.time() * 1000)),
               'operatorUid': get_user_id(), 'operatedObjId': _event_name_}
    try:
        client.bury_data_upload(payload)
    except ValueError:
        pass


def bury_data(event_name=None):
    def decorator(func):
        def wrapper(*args, **kwargs):
            _event_name_ = event_name
            if _event_name_ is None:
                _event_name_ = get_full_event_name(func.__name__)
            else:
                _event_name_ = get_full_event_name(_event_name_)
            client = OpenapiClient(endpoint)
            bury_thread_pool.submit_task(bury_data_upload_task, _event_name_, client)
            result = func(*args, **kwargs)
            return result

        return wrapper

    return decorator


def get_full_event_name(method_name) -> str:
    """
    Judging whether it is the sdk calling method according to the system parameters
    """
    if len(sys.argv) > 1:
        return f"py_client_{method_name}_cli"
    else:
        return f"py_client_{method_name}_sdk"


def get_user_id() -> str:
    try:
        token = get_token()
        if token is not None:
            return token.sso_uid
        else:
            return 'anonymous'
    except ValueError:
        return 'anonymous'



import yaml
import re
import os


class MetafileParser:
    def __init__(self, filename):
        self.filename = filename
        self.metafile = None

    def parse(self):
        with open(self.filename, 'r') as f:
            self.metafile = yaml.safe_load(f)
        return self.metafile

    def validate(self):
        if not self.metafile.get('Collections'):
            raise ValueError("Error: Collections field is missing or empty.")
        if not self.metafile.get('Models'):
            raise ValueError("Error: Models field is missing or empty.")
        collections = self.metafile.get('Collections')
        for collection in collections:
            if not collection.get('Name'):
                raise ValueError("Error: Collections Name field is missing or empty.")
            if not collection.get('License'):
                raise ValueError("Error: Collections License field is missing or empty.")
            if not collection.get('Code'):
                raise ValueError("Error: Collections Code field is missing or empty.")
            if not collection.get('Code')['URL']:
                raise ValueError("Error: Collections Code URL field is missing or empty.")
        models = self.metafile.get('Models')
        for model in models:
            if not model.get('Name'):
                raise ValueError("Error: Models Name field is missing or empty.")
            if not model.get('Results'):
                raise ValueError("Error: Models Results field is missing or empty.")
            results = model.get('Results')
            for result in results:
                if not result.get('Task'):
                    raise ValueError("Error: Models Results Task field is missing or empty.")
                if not result.get('Task'):
                    raise ValueError("Error: Models Results Dataset field is missing or empty.")
            if not model.get('Weights'):
                raise ValueError("Error: Models Weights field is missing or empty.")
        # Add more validation rules as needed

    def parse_and_validate(self) -> dict:
        self.parse()
        self.validate()
        print(f"parse metafile data:{self.metafile}")
        return self.metafile


def get_filename_from_url(url):
    if url.startswith("http://") or url.startswith("https://"):
        raise ValueError(f'Weights must be local path file')
    url.replace('\\', '/')
    url_array = url.split('/')
    if len(url_array) == 1:
        return url_array[0]
    else:
        return url_array[len(url_array) - 1]


def get_meta_payload(repository, private, meta_data):
    _name = repository
    _nickname = repository
    _collection = meta_data['Collections'][0]
    _license = _collection['License']
    _github = _collection['Code']['URL']
    _public_status = 0 if private else 1
    _models = [{"name": m['Name'], "weightName": get_filename_from_url(m['Weights']),
                "evaluations": [{"task": e['Task'], "dataset": e['Dataset']} for e in m['Results']]} for m in
               meta_data['Models']]

    payload = {"name": _name, "nickname": _nickname, "license": _license,
               "github": _github, "publicStatus": _public_status, "models": _models}

    print(f"convert to payload:{payload}")
    return payload



"""
model module constant define
"""
import os

model_openapi_url_prefix_dev = "https://dev.openxlab.org.cn/api/v1"
model_openapi_url_prefix_staging = "https://staging.openxlab.org.cn/api/v1"
model_openapi_url_prefix_prod = "https://openapi.openxlab.org.cn/api/v1"
model_url_prefix_dev = "http://10.1.100.130:10019"
model_cache_path = os.path.join(os.path.expanduser("~"), '.cache', 'model')
# temp token
token = "eyJ0eXBlIjoiSldUIiwiYWxnIjoiSFM1MTIifQ.eyJqdGkiOiI1OTcwOTEiLCJyb2wiOiJST0xFX0FETUlOIiwiaXNzIjoiT3BlblhMYWIiLCJpYXQiOjE2ODE5Njg4NzMsInBob25lIjoiIiwiYWsiOiI2cHFnOXprNmRteG9rZ2JnbHZvayIsImVtYWlsIjoiZG9uZ3hpYW96aHVhbmdAcGpsYWIub3JnLmNuIiwiZXhwIjoxNzEzNTA0ODczfQ.eFl8ZH9tDp-pcjY3wz6PeNBarJwhVx90qQ3h82Qvpf0hrcrdrQSBcI8AmEk2TZFpeViC6HBtXRxxGp2YLK1XkA"
endpoint = model_openapi_url_prefix_prod
paths = {
    'file_download_path': '/model-center/api/v1/cli/repository/getFileDownloadUrl',
    'meta_file_template_download_path': '/model-center/api/v1/cli/repository/getMetafileTemplateUrl',
    'create_repository_path': '/model-center/api/v1/cli/repository/createModelRepository',
    'update_repository_path': '/model-center/api/v1/cli/repository/updateRepositoryBaseInfo',
    'delete_repository_path': '/model-center/api/v1/cli/repository/deleteRepository',
    'query_models_path': '/model-center/api/v1/cli/repository/getRepositoryModelList',
    'update_upload_status_path': '/model-center/api/v1/cli/repository/updateFileUploadStatus',
    'query_model_repo_info_path': '/model-center/api/v1/cli/repository/queryModelRepoInfo',
    'get_upload_signature': "/upload-service/api/v1/getUploadSignature",
    'bury_upload': "/data-bury/api/v1/bury/collect",
}
# oss_endpoint = 'https://openmmlab-open.oss-cn-shanghai.aliyuncs.com'
oss_endpoint = 'https://oss-cn-shanghai.aliyuncs.com'
oss_bucket_domain_dev = 'openmmlab-open.oss-cn-shanghai.aliyuncs.com'
oss_bucket_domain_staging = 'openmmlab-open.oss-cn-shanghai.aliyuncs.com'
oss_bucket_domain_prod = 'xlab-model-center.oss-cn-shanghai.aliyuncs.com'
oss_bucket_domain = oss_bucket_domain_prod
oss_endpoint_domain = 'oss-cn-shanghai.aliyuncs.com'
oss_accelerate_domain = 'oss-accelerate.aliyuncs.com'
default_metafile_template_name = 'metafile.yaml'
river_pass_url_dev = 'http://106.14.134.80:10006/getFile'
river_pass_url_staging = 'http://riverpass.staging.openxlab.org.cn/getFile'
river_pass_url_prod = 'http://riverpass.openxlab.org.cn/getFile'
river_pass_url = river_pass_url_prod



from argparse import Namespace

from openxlab.types.command_type import BaseCommand
from openxlab.xlab.handler import user_token


class Token(BaseCommand):
    """Print token"""

    def get_name(self) -> str:
        return "token"

    def take_action(self, parsed_args: Namespace):
        print(user_token.get_jwt())



"""
upload model file|meta file|log file|readme file-cli
"""
from openxlab.types.command_type import *
from openxlab.model import upload


class Upload(BaseCommand):
    """Upload model-related files."""

    def get_name(self) -> str:
        return "upload"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = 'openxlab model upload [-h] -r MODEL_REPO [-ft FILE_TYPE] -s SOURCE [-t TARGET] {help} ...\n' \
                       'Example:\n' \
                       '> openxlab model upload -r username/model_repo_name -s metafile.yaml'
        parser.add_argument('-r', '--model-repo', required=True,
                            help='The name of model repository.[required]')
        parser.add_argument('-ft', '--file-type', type=str, default='metafile', required=False,
                            help='Upload file type, metafile/other, default metafile.')
        parser.add_argument('-s', '--source', type=str, required=True,
                            help='The path of the file to be uploaded. [required]')
        parser.add_argument('-t', '--target', type=str, required=False,
                            help='The path where the uploaded file is stored or the name of the replaced file.')
        parser.add_argument('-d', '--domain', type=str, required=False,
                            help='domain for uploading.')

    def take_action(self, parsed_args: Namespace) -> int:
        upload(parsed_args.model_repo, parsed_args.file_type, parsed_args.source, parsed_args.target, parsed_args.domain)
        return 0



import getpass

from openxlab.types.command_type import *
from openxlab.xlab.handler.user_token import *


class Config(BaseCommand):
    """Config Openxlab AKSK"""

    def get_name(self) -> str:
        return "config"

    def take_action(self, parsed_args: Namespace) -> int:
        ak = input("OpenXLab Access Key ID [NONE]: ")
        if len(ak) == 0:
            raise ValueError("access key id must not be empty")
        sk = getpass.getpass("OpenXLab Secret Access Key [None]: ")
        if len(sk) == 0:
            raise ValueError("secret access key must not be empty")
        user_config = UserConfig(ak, sk)
        user_config.store_to_local()
        return 0



import argparse
import os
import re
from urllib.parse import urlparse


def parse_url(url: str):
    o = urlparse(url)
    host = f"{o.scheme}://{o.hostname}"
    if o.port is not None:
        host += ":" + str(o.port)

    dataset_name = o.path.split("/")[-1]
    return host, dataset_name


def get_api_token_from_env():
    return os.environ.get("OPENDATALAB-API-TOKEN", "")


SYMBOLS = {
    'customary': ('B', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'),
    'customary_ext': ('byte', 'kilo', 'mega', 'giga', 'tera', 'peta', 'exa', 'zetta', 'iotta'),
    'iec': ('Bi', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi', 'Yi'),
    'iec_ext': ('byte', 'kibi', 'mebi', 'gibi', 'tebi', 'pebi', 'exbi', 'zebi', 'yobi'),
}


def bytes2human(n, format='%(value).1f%(symbol)s', symbols='customary'):
    """
    Convert n bytes into a human-readable string based on format.
    symbols can be either "customary", "customary_ext", "iec" or "iec_ext",
    see: http://goo.gl/kTQMs
    """

    n = int(n)
    if n < 0:
        raise ValueError("n < 0")
    symbols = SYMBOLS[symbols]
    prefix = {}
    for i, s in enumerate(symbols[1:]):
        prefix[s] = 1 << (i + 1) * 10
    for symbol in reversed(symbols[1:]):
        if n >= prefix[symbol]:
            value = float(n) / prefix[symbol]
            return format % locals()
    return format % dict(symbol=symbols[0], value=n)


def human2bytes(s):
    """
    Attempts to guess the string format based on default symbols
    set and return the corresponding bytes as an integer.
    When unable to recognize the format ValueError is raised.
    """
    init = s
    num = ""
    while s and s[0:1].isdigit() or s[0:1] == '.':
        num += s[0]
        s = s[1:]
    num = float(num)
    letter = s.strip()
    for name, sset in SYMBOLS.items():
        if letter in sset:
            break
    else:
        if letter == 'k':
            # treat 'k' as an alias for 'K' as per: http://goo.gl/kTQMs
            sset = SYMBOLS['customary']
            letter = letter.upper()
        else:
            raise ValueError("can't interpret %r" % init)
    prefix = {sset[0]: 1}
    for i, s in enumerate(sset[1:]):
        prefix[s] = 1 << (i + 1) * 10
    return int(num * prefix[letter])


def str2bool(v):
    """convert keywords like true to True, so do as False"""
    if isinstance(v, bool):
        return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')


def find_url(text: str):
    """get urls with re module"""
    urls = re.findall(r'(https?://[^"]+)', text)
    return urls


def replace_urls(text: str, urls: list):
    """replace url with given string to use rich package"""
    for url in urls:
        new_url = "[blue underline]" + url + "[/blue underline]"
        text = text.replace(url, new_url)
    return text


def highlight_urls(text: str):
    urls = find_url(text=text)
    new_text = replace_urls(text=text, urls=urls)
    return new_text



from argparse import ArgumentParser
from argparse import Namespace
from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import Type
from typing import Union


class BaseCommand:
    """Base class for commands.
    """
    sub_command_list = []

    def __init__(self) -> None:
        pass
        
    def get_name():
        """define command name"""
        pass
        
    def add_arguments(self, parser: ArgumentParser) -> None:
        """Add arguments required for each command.

        Args:
            parser:
                `ArgumentParser` object to add arguments
        """
        pass

    def take_action(self, parsed_args: Namespace) -> int:
        """Define action if the command is called.

        Args:
            parsed_args:
                `Namespace` object including arguments specified by user.

        Returns:
            Running status of the action.
            0 if this method finishes normally, otherwise 1.
        """

        raise NotImplementedError


import json
import sys

import requests

from openxlab.dataset.constants import odl_clientId
from openxlab.dataset.constants import uaa_url_prefix
from openxlab.xlab.handler.user_token import get_jwt
from openxlab.xlab.handler.user_token import get_token
from openxlab.xlab.handler.user_token import get_token_from_local


api_auth = "/api/v1/internal/auth"
clientId = odl_clientId
auth_url = uaa_url_prefix + api_auth



def get_auth_code(sso_uid):
    result = None
    if sso_uid:
        client_id = {'clientId': clientId}
        data = json.dumps(client_id)
        resp = requests.post(url=auth_url,
                             data=data,
                             headers={
                                 "Content-Type": "application/json",
                                 "Cookie": f"ssouid={sso_uid}",
                             }
                             )
        if resp.status_code == 200:
            result = resp.json()['data']['code']
    return result


def get_odl_token(ak, sk):
    status = get_jwt(ak, sk)
    token = get_token()
    sso_uid = token.sso_uid
    if sso_uid:
        auth_code = get_auth_code(sso_uid=sso_uid)

    if not auth_code:
        print(f"Error: Auth failure")
        sys.exit(1)
    return auth_code, sso_uid



"""
创建模型仓库
"""
import logging
import os
import re
from tqdm import tqdm
import requests

from openxlab.model.clients.openapi_client import OpenapiClient
from openxlab.model.common.constants import endpoint, token
from openxlab.model.common.meta_file_util import MetafileParser
from openxlab.model.common.bury import bury_data
from openxlab.model.handler.upload_file import upload_model_list


@bury_data("create_model")
def create(model_repo, source="metafile.yaml", private=False) -> None:
    """
    create model repository
    usage: cli & sdk
    """
    try:
        # split params
        username, repository = _split_repo(model_repo)
        client = OpenapiClient(endpoint, token)
        # parse & check metafile.yml
        print("Current directory:", os.getcwd())

        meta_parser = MetafileParser(source)
        meta_data = meta_parser.parse_and_validate()
        client.create_repository(repository, private, meta_data)
        print(f"repository:{repository} created successfully.")
        # upload file
        meta_file_model_list = meta_data['Models']
        remote_model_list = [{**d, "name": d["Name"], "weightName": os.path.basename(d['Weights'])} for d in meta_file_model_list]
        upload_model_list(client, repository, meta_file_model_list, remote_model_list, domain='bucket')
        print(f"file upload successfully.")
    except ValueError as e:
        print(f"Error: {e}")
        return


def _split_repo(model_repo) -> (str, str):
    """
    Split a full repository name into two separate strings: the username and the repository name.
    """
    # username/repository format check
    pattern = r'.+/.+'
    if not re.match(pattern, model_repo):
        raise ValueError("The input string must be in the format 'username/model_repo'")

    values = model_repo.split('/')
    return values[0], values[1]


def _parse_check():
    pass



from datetime import datetime
import pytz


def get_current_formatted_time(timezone='Asia/Shanghai') -> str:
    formatted_date = get_current_time(timezone).strftime("%Y-%m-%d %H:%M:%S")
    return formatted_date


def get_current_time(timezone='Asia/Shanghai') -> datetime:
    now_utc = datetime.now(tz=pytz.utc)
    tz = pytz.timezone(timezone)
    return now_utc.astimezone(tz)


def get_datetime_from_formatted_str(time_str, timezone='Asia/Shanghai') -> datetime:
    if len(time_str) == 0:
        raise ValueError("formatted time string must not be empty")
    timezone = pytz.timezone(timezone)
    return datetime.strptime(time_str, '%Y-%m-%d %H:%M:%S').replace(tzinfo=timezone)



""" 
commit message to dataset repository cli
"""
from openxlab.dataset.handler.commit_dataset_info import commit
from openxlab.types.command_type import *


class Commit(BaseCommand):
    """Commit local changes."""

    def get_name(self) -> str:
        return "commit"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = (
            'openxlab dataset commit [OPTIONS]\n\n'
            "Commit local changes.\n\n"
            'Example:\n'
            '> openxlab dataset commit --dataset-repo "username/dataset-repo-name" --commit-message "message"\n'
        )
        parser.add_argument(
            "-r",
            "--dataset-repo",
            type=str,
            required=True,
            help='The address of the dataset repository to commit.format: username/dataset-repo-name.[required]',
        )
        parser.add_argument(
            "-m",
            "--commit-message",
            type=str,
            required=True,
            help='The detail message to commit of the repository.[required]',
        )

    def take_action(self, parsed_args: Namespace) -> int:
        commit(dataset_repo=parsed_args.dataset_repo, commit_message=parsed_args.commit_message)

        return 0



""" 
create dataset repository-cli
"""
from openxlab.dataset.handler.create_dataset_repository import create_repo
from openxlab.dataset.utils import str2bool
from openxlab.types.command_type import *


class Create(BaseCommand):
    """Create a dataset repository."""

    def get_name(self) -> str:
        return "create"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = (
            'openxlab dataset create [OPTIONS]\n\n'
            "Create a dataset repository.\n\n"
            'Example:\n'
            '> openxlab dataset create --repo-name \"dataset_repo_name\"'
        )
        parser.add_argument(
            "--repo-name",
            required=True,
            help='The name of dataset repository.[required]',
        )
        parser.add_argument(
            "-p",
            "--private",
            type=str2bool,
            # default=False,
            # required=True,
            help='The visibility permission of repository.',
        )

    def take_action(self, parsed_args: Namespace) -> int:
        create_repo(repo_name=parsed_args.repo_name, private=parsed_args.private)

        return 0



""" 
download the repository of dataset
"""
from openxlab.dataset.handler.get_dataset_repository import get
from openxlab.types.command_type import *


class Get(BaseCommand):
    """Get the dataset repository."""

    def get_name(self) -> str:
        return "get"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.usage = (
            "openxlab dataset get [OPTIONS]\n\n"
            "Get the dataset repository.\n\n"
            "Example:\n"
            ">openxlab dataset get --dataset-repo \"username/dataset-repo-name\" --target-path \"/path/to/local/folder\""
        )
        parser.add_argument(
            "-r",
            "--dataset-repo",
            type=str,
            required=True,
            help="The address of dataset repository. format: username/dataset-repo-name.[required]",
        )
        parser.add_argument(
            "-t",
            "--target-path",
            type=str,
            help="The target local path to save the dataset repository. [optional]",
        )

    def take_action(self, parsed_args: Namespace) -> int:
        get(parsed_args.dataset_repo, parsed_args.target_path)

        return 0



import os


# 默认的配置路径
DEFAULT_CONFIG_DIR = os.path.join(
    os.path.expanduser("~"), ".openxlab"
)

 # 默认配置文件名称
DEFAULT_CLI_CONFIG_FILE_NAME = "config.json" 

DEFAULT_CLI_TOKEN_FILE_NAME = "token.json"

DEFAULT_CLI_DATASET_FILE_NAME = "dataset.json"

AK_ENV_NAME = "OPENXLAB_AK"
SK_ENV_NAME = "OPENXLAB_SK"

OPENXLAB_ENV_VALUE_PREFIX = "OPENXLAB_"



"""
update repository
"""
import re

from openxlab.model.clients.openapi_client import OpenapiClient
from openxlab.model.common.constants import endpoint, token
from openxlab.model.common.bury import bury_data


@bury_data()
def visibility(model_repo, private) -> None:
    """
    update repository visibility private -> public
    """
    try:
        # split params
        username, repository = _split_repo(model_repo)
        client = OpenapiClient(endpoint, token)
        client.update_repository(repository, private)
        print(f"repository:{repository} visibility update to public successfully.")
    except ValueError as e:
        print(f"Error: {e}")
        return


def _split_repo(model_repo) -> (str, str):
    """
    Split a full repository name into two separate strings: the username and the repository name.
    """
    # username/repository format check
    pattern = r'.+/.+'
    if not re.match(pattern, model_repo):
        raise ValueError("The input string must be in the format 'username/model_repo'")

    values = model_repo.split('/')
    return values[0], values[1]


def _parse_check():

    pass



from argparse import ArgumentParser
from argparse import Namespace
import inspect
import logging
import sys
from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import Type
from typing import Union

import openxlab
from openxlab.dataset import Dataset
from openxlab.demo_cmd import Demo
from openxlab.model.commands import Model
from openxlab.types.command_type import BaseCommand
from openxlab.xlab import Config
from openxlab.xlab import Version
from openxlab.xlab.commands.login_command import Login
from openxlab.xlab.commands.token_command import Token
from openxlab.xlab.handler.user_token import get_token


# 定义一级子命令
class AllCommand(BaseCommand):
    sub_command_list = [Version, Demo, Config, Dataset, Model, Login, Token]

    def take_action(self, parsed_args: Namespace) -> int:
        return 0


# 递归添加子命令
def _add_sub_commands_recur(
        main_parser: ArgumentParser, parent_parser: ArgumentParser, command_type: BaseCommand
) -> Dict[str, ArgumentParser]:
    subparsers = main_parser.add_subparsers()

    sub_commands = command_type.sub_command_list
    for command_type in sub_commands:
        command = command_type()
        command_name = command.get_name()

        subparser = subparsers.add_parser(
            command_name, parents=[parent_parser], help=inspect.getdoc(command_type)
        )
        command.add_arguments(subparser)

        # 通过是否实现 take_action 判定命令的默认行为
        if issubclass(command_type, BaseCommand) and getattr(BaseCommand, "take_action") is not getattr(command_type,
                                                                                                        "take_action"):
            subparser.set_defaults(handler=command.take_action)
        else:
            def _print_help(args: Namespace) -> None:
                subparser.print_help()

            subparser.set_defaults(handler=_print_help)

        # 递归添加子命令
        _add_sub_commands_recur(subparser, parent_parser, command_type)

    # 添加默认的 help 命令
    def _print_help(args: Namespace) -> None:
        main_parser.print_help()

    subparsers.add_parser("help", help="Show help message and exit.").set_defaults(
        handler=_print_help
    )
    return


def _get_parser(description: str = "") -> ArgumentParser:
    parent_parser = ArgumentParser(add_help=False)

    main_parser = ArgumentParser(description=description, parents=[parent_parser])

    _add_sub_commands_recur(main_parser, parent_parser, AllCommand)

    return main_parser


def _preprocess_argv(argv: List[str]) -> List[str]:
    # Some preprocess is necessary for argv because some subcommand includes space
    # (e.g. optuna study optimize, optuna storage upgrade, ...).
    argv = argv[1:] if len(argv) > 1 else ["help"]

    # No subcommand is found.
    return argv


def main():
    main_parser = _get_parser()
    argv = sys.argv
    preprocessed_argv = _preprocess_argv(argv)
    args = main_parser.parse_args(preprocessed_argv)

    logger = logging.getLogger("openxlab")
    try:
        return args.handler(args)
    except Exception as e:
        logger.exception(e)
        # if args.debug:
        #     logger.exception(e)
        # else:
        #     logger.error(e)
        #     # This code is required to show help for each subcommand.
        #     # NOTE: the first element of `preprocessed_argv` is command name.
        #     command_name_to_subparser[preprocessed_argv[0]].print_help()
        return 1


if __name__ == "__main__":
    main()



""" 
upload local file to dataset repository
"""
from openxlab.dataset.commands.utility import ContextInfo
from openxlab.dataset.io.upload import Uploader


def upload_file(dataset_repo: str, source_path: str, target_path=""):
    """
    Upload file from local to remote.

    Example:
        openxlab.dataset.upload_file(
            dataset_repo="username/dataset_repo_name",
            source_path="/path/to/local/folder",
            target_path="/raw/file"
        )

    Parameters:
        @dataset_repo String The address of dataset repository.
        @source_path String The local path of the file to upload.
        @target_path String The target path to upload file.
    """
    ctx = ContextInfo()
    client = ctx.get_client().get_api()
    parsed_ds_name = dataset_repo.replace("/", ",")
    uploader = Uploader(client, parsed_ds_name)
    uploader.upload_file(source_path, target_path)



import getpass

from openxlab.types.command_type import *
from openxlab.xlab.handler.user_config import clear_dataset_json
from openxlab.xlab.handler.user_config import get_dataset_path
from openxlab.xlab.handler.user_login import login
from openxlab.xlab.handler.user_token import *


class Login(BaseCommand):
    """Login Openxlab"""

    def get_name(self) -> str:
        return "login"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument('-r', '--relogin', required=False,  action='store_true',
                            help='force relogin via new AK, SK')

    def take_action(self, parsed_args: Namespace):
        re_login = parsed_args.relogin
        if os.path.exists(get_config_path()) and re_login is False:
            print("AK and SK have been configured. You can use the --relogin parameter to force a re-login.")
            return
        print("openxlab: You can find your Access key & Secrete Key in your browser here: "
              "https://sso.openxlab.org.cn/usercenter?tab=secret")
        # clear dataset.json if re_login
        if os.path.exists(get_dataset_path()) and re_login is True:
            clear_dataset_json()
        ak = input("openxlab: Paste your Access Key here: ")
        sk = getpass.getpass("openxlab: Paste your Secrete Key here: ")
        login(ak, sk, re_login)









from openxlab.config import version as config_version
from openxlab.types.command_type import *

from openxlab.xlab.commands.config_command import Config


class Version(BaseCommand):
    """Get Openxlab Version"""
    
    def get_name(self) -> str:
        return "version"

    def add_arguments(self, parser: ArgumentParser) -> None:
        pass

    def take_action(self, parsed_args: Namespace) -> int:
        print("OpenXLab %s" % config_version.version)
        return 0
    
